"""
This module provides a simple user interface for tables. Tables can be sent
to the shell, saved as tab separated value (TSV) files and pickled.

Classes
-------
TableFrame is the frame which contains the grid
MyGrid_simple is the grid
TableAttachDialog asks for output options

"""

import logging
import cPickle as pickle

import numpy as np
import wx
import wx.grid
# import wx.lib.mixins.grid

from .._wxutils import Icon, ID
from .._utils.basic import add_ext, loadtable



class MyGrid_simple(wx.grid.Grid):  # , wx.lib.mixins.grid.GridAutoEditMixin): #wx.grid.PyGridTableBase,
    def __init__(self, parent):
        wx.grid.Grid.__init__(self, parent)
        self.parent = parent
        # wx.grid.PyGridTableBase.__init__(self)
        # wx.lib.mixins.grid.GridAutoEditMixin.__init__(self)
        self.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
    def OnKeyDown(self, event=None):
        key = event.GetKeyCode()
        logging.debug("MyGrid_simple.OnKeyDown: %s" % key)
        if key == wx.WXK_TAB:
            if self.GetGridCursorCol() == self.GetNumberCols() - 1:
                y = self.GetGridCursorRow() + 1
                if y == self.GetNumberRows():
                    self.parent.AppendRow()
                self.SetGridCursor(y, 0)
                return
        elif key in [wx.WXK_BACK, wx.WXK_DELETE]:
            cols, rows = self.GetSelectedCols(), self.GetSelectedRows()
            for c in cols:
                self.DeleteCols(c, 1)
            for r in rows:
                self.DeleteRows(r, 1)
            if rows or cols:
                return  # do not select cell content if row/col is deleted
        event.Skip()
    def set(self, table):
        x = max(len(row) for row in table)
        y = len(table)
        # self.ClearGrid() #NOTENOUGH
        self.parent.SetColsRows(x, y)
        for j, row in enumerate(table):
            for i, item in enumerate(row):
                self.SetCellValue(j, i, unicode(item))
        # self.AutoSize()
    def load(self, path):
        table = loadtable(path)
        self.set(table)
    def get(self, dt=unicode,
            asdict=False, kdt=unicode):
        x = self.GetNumberCols()
        y = self.GetNumberRows()
        if asdict == False:
            table = []
            for i in range(y):
                row = []
                for j in range(x):
                    value = self.GetCellValue(i, j)
                    if value in ['', 'nan', 'NAN']:
                        value = np.NAN
                    else:
                        try:
                            value = dt(value)
                        except:
                            logging.warning(" MyGrid_simple could not convert %s to %s" % (value, dt))
                            value = np.NAN
                    row.append(value)
                table.append(row)
        else:
            table = {}
            if asdict == 'row':
                for j in range(x):
                    col = []
                    for i in range(1, y):
                        col.append(dt(self.GetCellValue(i, j)))
                    if len(col) == 1:
                        col = col[0]
                    table[kdt(self.GetCellValue(0, j))] = col
            elif asdict == 'col':
                for i in range(y):
                    row = []
                    for j in range(1, x):
                        row.append(dt(self.GetCellValue(i, j)))
                    if len(row) == 1:
                        row = row[0]
                    table[kdt(self.GetCellValue(i, 0))] = row
            else:
                raise NotImplementedError("asdict needs to be False, 'row', or 'col'")
        return table
    def resize(self, cols, rows):
        cols_now = self.GetNumberCols()
        if cols_now > cols:
            pass
        else:
            self.AppendCols(cols - cols_now)
        rows_now = self.GetNumberRows()
        if rows_now > rows:
            pass
        else:
            self.AppendRows(rows - rows_now)


class TableAttachDialog(wx.Dialog):
    """
    Dialog to set options for attaching table

    """
    dtypes_s = ['int', 'float', 'unicode']
    dtypes = [int, float, unicode]
    # built on wxPython Demo 'Dialog' Example
    def __init__(self, parent, default_name,
                 title="Attach Table",
                 message="Settings for attaching table to global namespace",
                 ID=-1, size=wx.DefaultSize,
                 pos=wx.DefaultPosition, style=wx.DEFAULT_DIALOG_STYLE,
                 useMetal=False,):
        # Instead of calling wx.Dialog.__init__ we precreate the dialog
        # so we can set an extra style that must be set before
        # creation, and then we create the GUI object using the Create
        # method.
        pre = wx.PreDialog()
        pre.SetExtraStyle(wx.DIALOG_EX_CONTEXTHELP)
        pre.Create(parent, ID, title, pos, size, style)

        # This next step is the most important, it turns this Python
        # object into the real wrapper of the dialog (instead of pre)
        # as far as the wxPython extension is concerned.
        self.PostCreate(pre)

        # This extra style can be set after the UI object has been created.
        if 'wxMac' in wx.PlatformInfo and useMetal:
            self.SetExtraStyle(wx.DIALOG_EX_METAL)


        # Now continue with the normal construction of the dialog
        # contents
        sizer = wx.BoxSizer(wx.VERTICAL)

        label = wx.StaticText(self, -1, message)
        sizer.Add(label, 0, wx.ALIGN_CENTRE | wx.ALL, 5)

        ## CONTROLS #####   #####   #####   #####   #####   #####

        rb = self.dt_ctrl = wx.RadioBox(self, -1, "dtype",
                                  wx.DefaultPosition, wx.DefaultSize,
                                  self.dtypes_s, 2, wx.RA_SPECIFY_COLS)
        sizer.Add(rb, 0, wx.ALIGN_CENTRE | wx.ALL, 5)

        # asdict
        cb = self.asdict_ctrl = wx.CheckBox(self, -1, "As Dictionary")
        cb.SetHelpText("Instead of exporting the table, export {key:list} dict " + \
                       "with first row/coumn providing keys")
        sizer.Add(cb, 0, wx.GROW | wx.ALIGN_CENTER_VERTICAL | wx.ALL, 5)
        self.Bind(wx.EVT_CHECKBOX, self.EvtCheckBox, cb)

        # asdict BOX
        sbox = wx.StaticBox(self, -1, "Dict Properties")
        bsizer = wx.StaticBoxSizer(sbox, wx.VERTICAL)
        rb = self.keyloc_ctrl = wx.RadioBox(self, -1, "Key Location",
                                    wx.DefaultPosition, wx.DefaultSize,
                                    ['Col 0', 'Row 0'], 2, wx.RA_SPECIFY_COLS)
        bsizer.Add(rb, 0, wx.ALIGN_CENTRE | wx.ALL, 5)
        rb = self.kdt_ctrl = wx.RadioBox(self, -1, "Key dtype",
                                  wx.DefaultPosition, wx.DefaultSize,
                                  self.dtypes_s, 2, wx.RA_SPECIFY_COLS)
        bsizer.Add(rb, 0, wx.ALIGN_CENTRE | wx.ALL, 5)
        sizer.Add(bsizer, 0, wx.GROW | wx.ALIGN_CENTER_VERTICAL | wx.ALL, 5)  # 1, wx.EXPAND|wx.ALL, 25)


        # name
        if default_name:
            hlp = "The name in global namespace which the table will " + \
                  "be assigned"
            box = wx.BoxSizer(wx.HORIZONTAL)

            label = wx.StaticText(self, -1, "Name:")
            label.SetHelpText(hlp)
            box.Add(label, 0, wx.ALIGN_CENTRE | wx.ALL, 5)

            text = self.name_ctrl = wx.TextCtrl(self, -1, default_name, size=(120, -1))
            text.SetHelpText(hlp)
            box.Add(text, 1, wx.ALIGN_CENTRE | wx.ALL, 5)

            sizer.Add(box, 0, wx.GROW | wx.ALIGN_CENTER_VERTICAL | wx.ALL, 5)
        else:
            self.name_ctrl = False


        # bottom
        line = wx.StaticLine(self, -1, size=(20, -1), style=wx.LI_HORIZONTAL)
        sizer.Add(line, 0, wx.GROW | wx.ALIGN_CENTER_VERTICAL | wx.RIGHT | wx.TOP, 5)

        btnsizer = wx.StdDialogButtonSizer()

#        if wx.Platform != "__WXMSW__":
#            btn = wx.ContextHelpButton(self)
#            btnsizer.AddButton(btn)

        btn = wx.Button(self, wx.ID_OK)
        btn.SetHelpText("Export the table to the shell")
        btn.SetDefault()
        btnsizer.AddButton(btn)

        btn = wx.Button(self, wx.ID_CANCEL)
        btn.SetHelpText("Return to the table view without exporting the table")
        btnsizer.AddButton(btn)
        btnsizer.Realize()

        sizer.Add(btnsizer, 0, wx.ALIGN_CENTER_VERTICAL | wx.ALL, 5)

        self.SetSizer(sizer)
        sizer.Fit(self)

    def EvtCheckBox(self, event):
        pass
    def GetName(self):
        if self.name_ctrl:
            name = self.name_ctrl.GetValue()
            return name
        else:
            return False
    def GetKwargs(self):
        kwargs = {'dt': self.dtypes[self.dt_ctrl.GetSelection()],
                  'kdt': self.dtypes[self.kdt_ctrl.GetSelection()], }
        if self.asdict_ctrl.GetValue():
            if self.keyloc_ctrl.GetSelection() == 0:
                kwargs['asdict'] = 'col'
            else:
                kwargs['asdict'] = 'row'
        else:
            kwargs['asdict'] = False
        return kwargs



class TableFrame(wx.Frame, wx.FileDropTarget):
    def __init__(self, parent, table, rows=3, cols=3, title="Table", **kwargs):
        wx.Frame.__init__(self, parent, -1, title=title, **kwargs)
        wx.FileDropTarget.__init__(self)
        self.parent_shell = parent
        # toolbar
        tb = self.CreateToolBar(wx.TB_HORIZONTAL)
        tb.SetToolBitmapSize(size=(32, 32))

        tb.AddLabelTool(ID.PYDOC_LOAD, "Load", Icon("tango/actions/document-open"))
        self.Bind(wx.EVT_TOOL, self.OnFileOpen, id=ID.PYDOC_LOAD)

        # tb.AddLabelTool(ID.RESIZE, "resize", Icon("mine.table"))
        # self.Bind(wx.EVT_TOOL, self.OnTableResize, id=ID.RESIZE)
        tb.AddLabelTool(ID.TABLE_SAVE, "Save", Icon("tango/actions/document-save"))
        self.Bind(wx.EVT_TOOL, self.OnTableSave, id=ID.TABLE_SAVE)

        tb.AddSeparator()
        tb.AddLabelTool(ID.ATTACH, "attach", Icon("actions/attach"))
        self.Bind(wx.EVT_TOOL, self.OnAttach, id=ID.ATTACH)

        # tb.AddSeparator()
        txt = wx.StaticText(tb, -1, "Columns:")
        sc = wx.SpinCtrl(tb, ID.TABLE_COL, "3", min=0, max=1000, initial=cols)
        tb.AddControl(txt)
        tb.AddControl(sc)
        self.sc_c = sc
        txt = wx.StaticText(tb, -1, "Rows:")
        sc = wx.SpinCtrl(tb, ID.TABLE_ROW, "3", min=0, max=1000, initial=rows)
        tb.AddControl(txt)
        tb.AddControl(sc)
        self.sc_r = sc
        self.Bind(wx.EVT_SPINCTRL, self.OnSpinCtrl, id=-1)

        # tb.AddControl()

        tb.Realize()

        # grid
        g = MyGrid_simple(self)
        g.CreateGrid(rows, cols)
        self.grid = g
        if table:
            g.set(table)

        self.Sizer = wx.BoxSizer()
        self.Sizer.Add(g, 1, wx.EXPAND)

        self.Show()

    def OnSpinCtrl(self, event=None):
        id = event.GetId()
        n = event.GetInt()
        if id == ID.TABLE_ROW:
            n_now = self.grid.GetNumberRows()
            if n > n_now:
                self.grid.AppendRows(n - n_now)
            else:
                self.grid.DeleteRows(n, n_now - n)
        elif id == ID.TABLE_COL:
            n_now = self.grid.GetNumberCols()
            if n > n_now:
                self.grid.AppendCols(n - n_now)
            else:
                self.grid.DeleteCols(n, n_now - n)
    def SetColsRows(self, cols, rows):
        if self.grid.GetNumberCols() > 0:
            self.grid.DeleteCols(0, -1)
        if self.grid.GetNumberRows() > 0:
            self.grid.DeleteRows(0, -1)
        self.grid.AppendCols(cols)
        self.grid.AppendRows(rows)
        self.sc_c.SetValue(cols)
        self.sc_r.SetValue(rows)
    def AppendRow(self):
        self.grid.AppendRows(1)
        n = self.grid.GetNumberRows()
        self.sc_r.SetValue(n)
    def OnDropFiles(self, x, y, filenames):
        logging.debug("OnDropFiles: %s" % str(filenames))
        if type(filenames) == list:
            filename = filenames[0]
        else:
            filename = filenames
        self.load(filename)
    def OnTableResize(self, event=None):
        # dialog = wx.Dialog(self)
        logging.debug("OnTableResize")
        cols, rows = (5, 5)
        self.grid.resize(cols, rows)
    def load(self, path):
        table = loadtable(path)
        self.grid.set(table)
        self.SetTitle(path)
    def OnFileOpen(self, event=None):
        dialog = wx.FileDialog(self, "Select Table File",
                               style=wx.FD_OPEN)
        if dialog.ShowModal() == wx.ID_OK:
            path = dialog.GetPath()
            self.load(path)
    def OnTableSave(self, event=None):
        dialog = wx.FileDialog(self, "Save Table",
                               wildcard="Pickle (*.pickled)|*.pickled|" + \
                                        "Tab Separated Values (*.txt)|*.txt",
                               style=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT)
        path = None
        while path == None:
            if dialog.ShowModal() == wx.ID_OK:
                path = dialog.GetPath()
                file_type = dialog.GetFilterIndex()
                path = add_ext(path, ['pickled', 'txt'][file_type])
            else:
                return

        if file_type == 0:  # pickle
            dialog = TableAttachDialog(self, False, title="Pickle Table",
                                       message="Settings for pickling table:")
            dialog.CenterOnScreen()
            if dialog.ShowModal() == wx.ID_OK:
                table = self.grid.get(**dialog.GetKwargs())
                with open(path, 'wb') as fid:
                    pickle.dump(table, fid)
        elif file_type == 1:  # TSV
            with open(path, 'w') as f:
                for row in self.grid.get():
                    f.write('\t'.join(row) + '\n')
    def OnAttach(self, event=None, name=None):
        g = self.parent_shell.global_namespace
        if not name:
            default_name = 't'; i = 1
            while default_name in g:
                default_name = 't%s' % i; i += 1
            dialog = TableAttachDialog(self, default_name)
            dialog.CenterOnScreen()
#            dialog = wx.TextEntryDialog(self, "Name for Table", "Name",
#                                        default_name)
            repeat = True
            while repeat:
                i = dialog.ShowModal()
                if i == wx.ID_OK:
                    repeat = False
                    name = dialog.GetName()
                    if name in g:
                        dialog2 = wx.MessageDialog(self, "Name '%s' is already present in global namespace. Overwrite?" % name,
                                                   "uga", wx.YES_NO | wx.NO_DEFAULT)
                        overwrite = dialog2.ShowModal()
                        if not overwrite:
                            repeat = True
                else:
                    return
            self.parent_shell.shell_message('%s = <table>' % name,
                                            internal_call=True)
            g[name] = self.grid.get(**dialog.GetKwargs())
            self.Close()




"""Modified PyShell class"""

from datetime import datetime
import inspect
import logging
import os
import cPickle as pickle
import re
import string
import sys
import tempfile
import types
import webbrowser

import wx
import wx.stc
import wx.lib.colourselect
import wx.py.shell
from wx.py.frame import ID_AUTO_SAVESETTINGS, ID_COPY_PLUS, ID_SAVEHISTORY

import matplotlib as mpl
import matplotlib.pyplot as plt

from .. import __version__
from .. import fmtxt
from .._utils import ui
from .._utils import print_funcs
from .._wxgui.mpl_canvas import CanvasFrame
from .._wxutils import droptarget, Icon, ID
from .about_dialog import AboutFrame
from .help import HelpFrame
from .mpl_tools import PyplotManager
from .preferences_dialog import PreferencesDialog
from .py_editor import PyEditor
from .table import TableFrame


history_session_hdr = "#  Eelbrain Session:  %s"
_punctuation = string.punctuation.replace('.', '').replace('_', '')


def is_py_char(char):
    return (char.isalnum() or char == '_')


def is_py_varname(name):
    a = re.match('^[a-zA-Z_]', name)
    b = re.match('[a-zA-Z0-9_]', name)
    return a and b


# modify wx.py introspection to take into account __wrapped__
from wx.py.introspect import getConstructor
def getBaseObject(obj):
    """Return base object and dropSelf indicator for an object."""
    if inspect.isbuiltin(obj):
        # Builtin functions don't have an argspec that we can get.
        dropSelf = 0
    elif inspect.ismethod(obj):
        # Get the function from the object otherwise
        # inspect.getargspec() complains that the object isn't a
        # Python function.
        try:
            if obj.im_self is None:
                # This is an unbound method so we do not drop self
                # from the argspec, since an instance must be passed
                # as the first arg.
                dropSelf = 0
            else:
                dropSelf = 1
            obj = obj.im_func
        except AttributeError:
            dropSelf = 0
    elif inspect.isclass(obj):
        # Get the __init__ method function for the class.
        constructor = getConstructor(obj)
        if constructor is not None:
            obj = constructor
            dropSelf = 1
        else:
            dropSelf = 0
    elif callable(obj):
        # Get the __call__ method instead.
        try:
            obj = obj.__call__.im_func
            dropSelf = 1
        except AttributeError:
            dropSelf = 0
    else:
        dropSelf = 0

    #  MY MOD
    obj = getattr(obj, '__wrapped__', obj)
    # END MY MOD

    return obj, dropSelf
wx.py.introspect.getBaseObject = getBaseObject


# subclass Shell in order to set some custom properties
class Shell(wx.py.shell.Shell):
    def __init__(self, *args, **kwargs):  # Leads to recursion crash
        self.exec_mode = 0  # counter to determine whether other objects than
        # the shell itself are writing to writeOut
        self.has_moved = False  # keeps track whether any entity has written
        # to the shell in exec_mode
        super(Shell, self).__init__(*args, **kwargs)

    def writeOut(self, message):
        """

        sep = False: adds linebreaks at the top and at the bottom of message

        ascommand = False: adds the prompt in front of the message to mimmick
                   command
        """
        if self.exec_mode:
            message = unicode(message)
            message = self.fixLineEndings(message)

            start = self.promptPosStart  # start des prompt >>>
            end = self.promptPosEnd  # end   des prompt >>>
#            current = self.CurrentPos    # caret position
#            last = self.GetLastPosition()# end of the text
#            logging.debug("WRITEOUT POS: start %s; end %s; current %s; last %s"%(start, end, current, last))

#            self.SetCurrentPos(end)#start)
#            self.SetAnchor(start)
            self.SetSelection(start, end)
            self.ReplaceSelection(message)
            self.has_moved = True
#            self.AddText(message)

            new_start = self.GetCurrentPos()
#            new_end = new_start + (end-start)
            self.promptPosStart = new_start
            self.promptPosEnd = new_start  # new_end
#
#            self.SetCurrentPos(new_end)
#            self.SetAnchor(new_end)
        else:
            # same as done by:  wx.py.shell.Shell.writeOut(self, message)
            self.write(message)

    def writeErr(self, message):
        # TODO: color=(0.93359375, 0.85546875, 0.3515625)
#        ls = os.linesep
#        if message != ls:
#            message = message.replace(ls, ls+'!   ')
        message = message.replace('"""', '"')  # some deprecation errors contain """ messing up shell colors
        self.writeOut(message)

    def start_exec(self):
        self.exec_mode += 1

    def end_exec(self):
        self.exec_mode -= 1
        if self.exec_mode == 0 and self.has_moved:
            self.prompt()
            self.has_moved = False

    def showIntro(self, text=''):
        """Display introductory text in the shell."""
        if text:
            self.write(text)
        try:
            if self.interp.introText:
                if text and not text.endswith(os.linesep):
                    self.write(os.linesep)
                # modified to avoid printing invalid instructions
                for line in self.interp.introText.splitlines():
                    if line.startswith('Type "help"'):
                        pass
                    else:
                        self.write(line)
                # end modification
        except AttributeError:
            pass


class ShellFrame(wx.py.shell.ShellFrame):
    bufferOpen = 1  # Dummy attr so that py.frame enables Open menu
    bufferNew = 1  # same for New menu command
    bufferClose = 1  # same for Close menu command (handled by OnFileClose)
    def __init__(self, parent=None, app=None, title='Eelbrain Shell'):

    # --- set up PREFERENCES ---
        self.config = config = wx.Config("Eelbrain")
        std_paths = wx.StandardPaths.Get()

        # redirect stdio for debugging
        if app is not None:
            redirect = config.ReadBool('Debug/Redirect', False)
            if redirect:
                filename = config.Read('Debug/Logfile') or None
                app.RedirectStdio(filename)

        # override pyshell defaults
        if not config.HasEntry('Options/AutoSaveSettings'):
            config.WriteBool('Options/AutoSaveSettings', True)
        if not config.HasEntry('Options/AutoSaveHistory'):
            config.WriteBool('Options/AutoSaveHistory', True)

        # get pyshell dataDir
        config_dir = os.path.join(std_paths.GetUserConfigDir(), 'Eelbrain')
        if not os.path.exists(config_dir):
            os.mkdir(config_dir)

        # make sure startup file exists
        startup_file = os.path.join(config_dir, 'startup')
        if not os.path.exists(startup_file):
            with open(startup_file, 'w') as fid:
                fid.write("# Eelbrain startup script\n")

        # http://wiki.wxpython.org/FileHistory
        self.filehistory = wx.FileHistory(10)
        self.filehistory.Load(config)

    # --- SHELL initialization ---
        # put my Shell subclass into wx.py.shell
        wx.py.shell.Shell = Shell
        wx.py.shell.ShellFrame.__init__(self, parent=parent, title=title,
                                        config=config, dataDir=config_dir)

        self.SetStatusText('Eelbrain %s' % __version__)

        droptarget.set_for_strings(self.shell)

        # attr
        self.global_namespace = self.shell.interp.locals
        self.editors = []
        self.active_editor = None  # editor last used; updated by Editor.OnActivate and Editor.__init__
        self.tables = []
        self.experiments = []  # keep track of ExperimentFrames
        self._attached_items = {}
        self.help_viewer = None

        # load history/configuration
        if len(self.shell.history):
            self.shell.history.pop(-1)
        self.LoadSettings()
        now = datetime.now()
        info = history_session_hdr % now.isoformat(' ')[:16]
        self.shell.history.insert(0, info)

    # child windows
        x_min, y_min, x_max, y_max = wx.Display().GetGeometry()
        self.P_mgr = PyplotManager(self, pos=(x_max - 100, y_min + 22 + 4))

    # --- MENUS ---
        # wx 2.8 somehow does not manage to access the 'window' and 'help'
        # menus; this works in 2.9

        def get_menu_Id(name):
            for Id in xrange(self.menuBar.GetMenuCount()):
                if self.menuBar.GetMenuLabel(Id) == name:
                    return Id

        # for debugging:
        # create a name->Id dict for the current menus
        # (created by wx.py.frame.Frame.__createMenus() )
#        self.menu_names = {self.menuBar.GetMenuLabel(i): i for i in
#                           xrange(self.menuBar.GetMenuCount())}

    # recent menu
        recent_menu = self.recent_menu = wx.Menu()
        self.filehistory.UseMenu(recent_menu)
        self.filehistory.AddFilesToMenu()
        self.recent_menu_update_icons()

        self.Bind(wx.EVT_MENU_RANGE, self.OnRecentItemLoad, id=wx.ID_FILE1, id2=wx.ID_FILE9)

        # add menu item (can only add it to one menu apparently)
        help_txt = "Load an experiment or Python script from a list of recently used items"
        self.fileMenu.InsertMenu(0, wx.ID_ANY, 'Recent Files', recent_menu, help_txt)

    # preferences menu
        if wx.Platform == '__WXMAC__':
            app = wx.GetApp()
            ID_PREFERENCES = app.GetMacPreferencesMenuItemId()
        else:  # although these seem to be the same -- 5022
            ID_PREFERENCES = wx.ID_PREFERENCES
        self.optionsMenu.Append(ID_PREFERENCES, "&Preferences...\tCtrl+,")
        self.Bind(wx.EVT_MENU, self.OnPreferences, id=wx.ID_PREFERENCES)

    # add "Open Examples" to file menu
        ex_root = __file__
        for _ in xrange(3):
            ex_root, _ = os.path.split(ex_root)
        self._examples_root = os.path.join(ex_root, 'examples')
        self.fileMenu.Insert(3, ID.SHOW_EXAMPLES, "Open Example...")
        self.Bind(wx.EVT_MENU, self.OnShowExamples, id=ID.SHOW_EXAMPLES)

    # add "Open History" to file menu
        m = self.historyFileMenu = wx.Menu()

        # collect history by date
        self._history_items = {}
        i_stop = 0
        header = history_session_hdr % ""
        header_len = len(header)
        for i, line in enumerate(self.shell.history[1:], 1):
            if line.startswith(header):
                i_start = i - 1
                n_items = i_start - i_stop
                if n_items:
                    history = self.shell.history[i_start:i_stop:-1]
                    txt = os.linesep.join(history)
                    n_lines = txt.count(os.linesep) + 1
                    name = line[header_len:]
                    label = name + ' (%i lines)' % n_lines
                    id_ = wx.NewId()
                    m.Append(id_, label, "Open history from %s." % label)
                    self.Bind(wx.EVT_MENU, self.OnOpenHistory, id=id_)
                    self._history_items[id_] = (name, txt)
                i_stop = i

        m.PrependSeparator()
        m.Prepend(ID.OPEN_HISTORY_CURRENT, "Current Session", "Open the "
                  "history for the current session in a Python document.")
        m.Prepend(ID.OPEN_HISTORY, "Entire History", "Open the entire history "
                  "as a new Python document.")
        self.fileMenu.InsertMenu(1, ID.OPEN_HISTORY, "History", m, "Open the "
                                 "command history in a Python script editor.")
        self.Bind(wx.EVT_MENU, self.OnOpenHistory, id=ID.OPEN_HISTORY_CURRENT)
        self.Bind(wx.EVT_MENU, self.OnOpenHistory, id=ID.OPEN_HISTORY)

    # edit menu
        if wx.Platform == '__WXMAC__':
            id_ = self.editMenu.FindItem('Redo \t')
            item = self.editMenu.FindItemById(id_)
            item.SetItemLabel('Redo \tctrl-shift-z')

    # Exec Menu
        m = self.execMenu = wx.Menu()
        m.Append(ID.EXEC_SELECTION, "Selection or Line  \tCtrl+Enter",
                 "Execute the currently selected code or the current line.")
        m.Append(ID.EXEC_DOCUMENT, "Document", "Execute the front most Python "
                 "document.")
        m.Append(ID.EXEC_DOCUMENT_FROM_DISK, "Document from Disk  \tCtrl+E",
                 "Save and execute the Python document from disk.")
        self.menuBar.Insert(get_menu_Id('&Help'), m, "Exec")

        self.Bind(wx.EVT_MENU, self.OnExecSelection, id=ID.EXEC_SELECTION)
        self.Bind(wx.EVT_MENU, self.OnExecDocument, id=ID.EXEC_DOCUMENT)
        self.Bind(wx.EVT_MENU, self.OnExecDocumentFromDisk,
                  id=ID.EXEC_DOCUMENT_FROM_DISK)

        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateMenu, id=ID.EXEC_SELECTION)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateMenu, id=ID.EXEC_DOCUMENT)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateMenu,
                  id=ID.EXEC_DOCUMENT_FROM_DISK)

    # MNE Menu
        # (without requiring mne-python to be installed)
        try:
            import mne
            m = self.mneMenu = wx.Menu()

            self.mne_cmds = {}
            for cmd in sorted(dir(mne.gui)):
                if cmd.startswith('_'):
                    continue

                # check that the command has no obligatory parameters
                func = getattr(mne.gui, cmd)
                argspec = inspect.getargspec(func)
                args = argspec.args
                if args:
                    defaults = argspec.defaults
                    if (defaults is None) or (len(args) > len(defaults)):
                        continue

                # add menu item
                Id = wx.NewId()
                self.mne_cmds[Id] = cmd
                name = cmd.capitalize()
                m.Append(Id, name, "Open the mne %s GUI" % name)
                self.Bind(wx.EVT_MENU, self.OnOpenMneGui, id=Id)

            self.menuBar.Insert(get_menu_Id("&Help"), m, "MNE")
        except:
            pass

    # WINDOW MENU
        Id = get_menu_Id("&Window")
        if Id is None:
            m = self.windowMenu = wx.Menu()  # title="&Window")
            m.SetTitle("&Window")
            Id_help = get_menu_Id('&Help')
            self.menuBar.Insert(Id_help, m, 'Window')
        else:
            m = self.windowMenu

        # name is used below in OnOpenWindowMenu to recognize Window menu
        self.Bind(wx.EVT_MENU_OPEN, self.OnOpenWindowMenu)

        m.Append(ID.PYPLOT_DRAW, '&Pyplot Draw  \tCtrl+P', 'Draw all figures.')
        self.Bind(wx.EVT_MENU, self.OnPyplotDraw, id=ID.PYPLOT_DRAW)

        m.Append(ID.P_MGR, "Pyplot Manager",
                 "Toggle Pyplot Manager Panel.")  # , wx.ITEM_CHECK)
        self.Bind(wx.EVT_MENU, self.OnTogglePyplotMgr, id=ID.P_MGR)

        m.Append(ID.PYPLOT_CLOSEALL, "Close All Plots",
                 "Closes all Matplotlib plots.")
        self.Bind(wx.EVT_MENU, self.OnCloseAllPlots, id=ID.PYPLOT_CLOSEALL)

        # shell resizing
        m.AppendSeparator()
        m.Append(ID.FOCUS_SHELL, "Focus Shell Prompt \tCtrl+L", "Raise the "
                 "shell and bring the caret to the end of the prompt")
        self.Bind(wx.EVT_MENU, self.OnFocusPrompt, id=ID.FOCUS_SHELL)

        m.Append(ID.SIZE_MAX, 'Maximize Shell',
                 "Expand the shell to use the full screen height.")
        self.Bind(wx.EVT_MENU, self.OnResize, id=ID.SIZE_MAX)

        m.Append(ID.SIZE_MAX_NOTITLE, 'Maximize Hide Title',
                 "Expand the shell and hide the title bar.")
        self.Bind(wx.EVT_MENU, self.OnResize, id=ID.SIZE_MAX_NOTITLE)

        m.Append(ID.SIZE_MIN, 'Mini-Shell', "Resize the Shell to a small "
                 "window (e.g. for use as pocket calculator).")
        self.Bind(wx.EVT_MENU, self.OnResize, id=ID.SIZE_MIN)

        # section with all open windows
        m.AppendSeparator()
        item = m.Append(self.GetId(), "Shell", "Bring shell to the front.")
        self.Bind(wx.EVT_MENU, self.OnWindowMenuActivateWindow, item)
        self.windowMenuWindows = {self.GetId(): self}
        self.windowMenuMenuItems = {}

        # clear shell: this can be done with edit->empty buffer

    # EDIT MENU
        m = self.editMenu
        m.AppendSeparator()
        m.Append(ID.COMMENT, "Comment Line \tCtrl+/", "Comment in/out current "
                 "line")
        self.Bind(wx.EVT_MENU, self.OnComment, id=ID.COMMENT)
        m.Append(ID.DUPLICATE, "Duplicate Commands \tCtrl-D")
        self.Bind(wx.EVT_MENU, self.OnDuplicate, id=ID.DUPLICATE)
        m.Append(ID.DUPLICATE_WITH_OUTPUT, "Duplicate Plus \tCtrl-Shift-D")
        self.Bind(wx.EVT_MENU, self.OnDuplicate, id=ID.DUPLICATE_WITH_OUTPUT)

        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateMenu, id=ID.COMMENT)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateMenu, id=ID.DUPLICATE)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateMenu, id=ID.DUPLICATE_WITH_OUTPUT)


    # INSERT MENU
        m = self.insertMenu = wx.Menu()
        m.Append(ID.INSERT_Color, "Color", "Insert color as (r, g, b)-tuple.")
#        self.Bind(wx.EVT_MENU, self.OnInsertColor, id=ID.INSERT_Color)
#        this function only works for the colorpicker linked to the button

        # path submenu
        m = self.pathMenu = wx.Menu()
        m.Append(ID.INSERT_Path_file, "File",
                 "Insert path to an existing File (You can also simply "
                 "drag a file on the shell or an editor).")
        m.Append(ID.INSERT_Path_dir, "Directory",
                 "Insert path to an existing Directory (You can also "
                 "simply drag a file on the shell or an editor).")
        m.Append(ID.INSERT_Path_new, "New",
                 "Insert path to a non-existing object.")
        self.insertMenu.AppendSubMenu(m, "Path", "Insert a path as string.")

        Id_view = get_menu_Id('&View')
        self.menuBar.Insert(Id_view, self.insertMenu, "Insert")
        self.Bind(wx.EVT_MENU, self.OnInsertPath_File, id=ID.INSERT_Path_file)
        self.Bind(wx.EVT_MENU, self.OnInsertPath_Dir, id=ID.INSERT_Path_dir)
        self.Bind(wx.EVT_MENU, self.OnInsertPath_New, id=ID.INSERT_Path_new)

    # OPTIONS menu
        # remove the conflicting keyboard shortcut
        m = self.optionsMenu.FindItemByPosition(0)
        sm = m.GetSubMenu()
        i = sm.FindItemByPosition(3)
        i.SetItemLabel(i.ItemLabel[:-13])

    # HELP menu
        if wx.__version__ >= '2.9':
            m = self.helpMenu
            m.AppendSeparator()
        else:
            m = wx.Menu()
            Id = self.menuBar.GetMenuCount()
            self.menuBar.Insert(Id, m, "Online Help")


        m.Append(ID.HELP_EELBRAIN, "Eelbrain", "Open the Eelbrain "
                 "documentation pages in an external browser.")
        self.Bind(wx.EVT_MENU, self.OnHelpExternal, id=ID.HELP_EELBRAIN)

        m.Append(ID.HELP_PYTHON, "Python", "Open the official Python "
                 "documentation page in an external browser.")
        self.Bind(wx.EVT_MENU, self.OnHelpExternal, id=ID.HELP_PYTHON)

        m.Append(ID.HELP_PDB, "   pdb (Python Debugger)", "Open the Python "
                 "Debugger documentation page in an external browser.")
        self.Bind(wx.EVT_MENU, self.OnHelpExternal, id=ID.HELP_PDB)

        m.Append(ID.HELP_MPL, "Matplotlib (Pyplot)", "Open the Matplotlib "
                 "Pyplot reference page in an external browser.")
        self.Bind(wx.EVT_MENU, self.OnHelpExternal, id=ID.HELP_MPL)

        m.Append(ID.HELP_MDP, "mdp",
                 "Open the mdp Documentation page in an external browser.")
        self.Bind(wx.EVT_MENU, self.OnHelpExternal, id=ID.HELP_MDP)


    # --- TOOLBAR ---
        tb = self.CreateToolBar(wx.TB_HORIZONTAL)
        tb.SetToolBitmapSize(size=wx.Size(32, 32))

        # windows/tools
        tb.AddLabelTool(wx.ID_HELP, "Help", Icon("tango/apps/help-browser"),
                        shortHelp="Open the Help-Viewer",
                        longHelp="Open the Help-Viewer")
        self.Bind(wx.EVT_TOOL, self.OnHelpViewer, id=wx.ID_HELP)

        tb.AddLabelTool(ID.P_MGR, "Pyplot Manager",
                        Icon("tango/mimetypes/x-office-presentation"),
                        shortHelp="Toggle display of the Pyplot manager tool")
        self.Bind(wx.EVT_TOOL, self.OnTogglePyplotMgr, id=ID.P_MGR)
        tb.AddSeparator()

        # File Operations
        tb.AddLabelTool(wx.ID_OPEN, "Load",
                        Icon("tango/actions/document-open"), shortHelp="Load "
                        "a Python document (*.py)")

        tb.AddLabelTool(ID.EXEC_FILE, "Exec", Icon("documents/pydoc-openrun"),
                        shortHelp="Run an existing Python script (without "
                        "opening it in an editor)")
        self.Bind(wx.EVT_TOOL, self.OnExecFile, id=ID.EXEC_FILE)

        tb.AddLabelTool(wx.ID_NEW, "New .Py Document", Icon("documents/pydoc-new"),
                        shortHelp="Open a new Python script editor")

        tb.AddLabelTool(ID.TABLE, "Table", Icon("documents/table"),
                        shortHelp="Open a new table")
        self.Bind(wx.EVT_TOOL, self.OnTableNew, id=ID.TABLE)
        tb.AddSeparator()

        # inserting (these controls have no shortHelp)
        fn_ctrl = wx.Choice(tb, wx.ID_FIND, (100, 50), choices=['File', 'Dir', 'New'])
        self.Bind(wx.EVT_CHOICE, self.OnInsertPath, id=wx.ID_FIND)
        tb.AddControl(fn_ctrl)
        b = wx.lib.colourselect.ColourSelect(tb, -1, "Color", (0, 255, 0),
                                             size=wx.DefaultSize)
        b.Bind(wx.lib.colourselect.EVT_COLOURSELECT, self.OnInsertColor)
        self.color_selector = b
        # bind the 'insert color' menu entry
        self.Bind(wx.EVT_MENU, self.color_selector.Command, id=ID.INSERT_Color)
        tb.AddControl(b)
        # tb.AddLabelTool(ID.COLOUR_CHOOSER, "Color", Icon('apps.preferences-desktop-locale'))
        # self.Bind(wx.EVT_TOOL, self.OnSelectColourAlt, id=ID.COLOUR_CHOOSER)

        if wx.__version__ >= '2.9':
            tb.AddStretchableSpace()
        else:
            tb.AddSeparator()

        # TODO: clear only the last command + output
        tb.AddLabelTool(ID.CLEAR_TERMINAL, "Clear Text", Icon("tango/actions/edit-clear"),
                        shortHelp="Clear all text from the terminal")
        self.Bind(wx.EVT_TOOL, self.OnClearTerminal, id=ID.CLEAR_TERMINAL)

        tb.AddLabelTool(wx.ID_EXIT, "Quit", Icon("tango/actions/system-log-out"),
                        shortHelp="Quit eelbrain")
        self.Bind(wx.EVT_TOOL, self.OnQuit, id=wx.ID_EXIT)

        tb.Realize()


    # --- Finalize ---

        # add commands to the shell
        self.global_namespace['attach'] = self.attach
        self.global_namespace['detach'] = self.detach
        self.global_namespace['help'] = self.help_lookup
        if wx.__version__ < '2.9':
            self.global_namespace['cd'] = self.curdir

        for name in ('printdict', 'printlist', 'dicttree'):
            self.global_namespace[name] = getattr(print_funcs, name)

        # other Bindings
        self.Bind(wx.EVT_MAXIMIZE, self.OnMaximize)
        self.Bind(wx.EVT_ACTIVATE, self.OnActivate)
        # my shell customization
        self.ApplyStyle()
        self.Resize(ID.SIZE_MAX)
        # icon
        if sys.platform != 'darwin':
            self.eelbrain_icon = Icon('eelbrain', asicon=True)
            self.SetIcon(self.eelbrain_icon)
            self.Bind(wx.EVT_CLOSE, self.OnDestroyIcon)


        # add help text from wx.py.shell
        text = wx.py.shell.HELP_TEXT
        self.__doc__ = text

        self.preferencesDialog = None

    def ApplyStyle(self):
        "reapply the layout to all editwindows"
#        if 'wxMac' in wx.PlatformInfo:
        self._style = {'times'     : 'Lucida Grande',
                       'mono'      : self.config.Read('font', 'Monaco'),  # 'Courier New'
                       'helv'      : 'Geneva',
                       'other'     : 'new century schoolbook',
                       'size'      : int(self.config.Read('font size', '13')),
                       'lnsize'    : 16,
                       'forecol'   : self.config.Read('font color', '#FFFF00'),
                       'backcol'   : '#F0F0F0',  # '#000010',
                       'calltipbg' : '#101010',
                       'calltipfg' : '#BBDDFF',
                       }

        self.ApplyStyleTo(self.shell)
        for ed in self.editors:
            if hasattr(ed, 'editor'):  # check if alife
                self.ApplyStyleTo(ed.editor.window)

    def ApplyStyleTo(self, obj):
        """
        see
         - wx.py.editwindow.EditWindow.setStyles
         - wx.py.shell.editwindow.EditWindow.__config()

        """
        obj.SetCaretWidth(2)

        obj.StyleClearAll()
        FACES = self._style

        obj.setStyles(FACES)  # this calls wx.py.editwindow.EditWindow.setStyles
        obj.CallTipSetBackground(FACES['calltipbg'])
        obj.CallTipSetForeground(FACES['calltipfg'])

        obj.StyleSetSpec(wx.stc.STC_STYLE_DEFAULT, "fore:%(forecol)s" % FACES)  # ,back:#666666")
        obj.StyleSetSpec(wx.stc.STC_P_DEFAULT, "fore:#000000,face:%(mono)s" % FACES)  # '\' space
        obj.StyleSetSpec(wx.stc.STC_P_NUMBER, "fore:#0000FF")
        obj.StyleSetSpec(wx.stc.STC_STYLE_CONTROLCHAR, "fore:#FF0000")
        obj.StyleSetSpec(wx.stc.STC_P_STRING, "fore:#FF0000")
        obj.StyleSetSpec(wx.stc.STC_P_CHARACTER, "fore:#0000FF")
        obj.StyleSetSpec(wx.stc.STC_P_OPERATOR, "fore:#000010,face:%(mono)s" % FACES)
        obj.StyleSetSpec(wx.stc.STC_P_DECORATOR, "fore:#FF00FF")
        obj.StyleSetSpec(wx.stc.STC_P_WORD, "fore:#FF00FF")
        obj.StyleSetSpec(wx.stc.STC_STYLE_MAX, "fore:#0000FF")
        comment = "fore:#008811"
        obj.StyleSetSpec(wx.stc.STC_P_COMMENTLINE, comment)
        obj.StyleSetSpec(wx.stc.STC_P_TRIPLEDOUBLE, "%s,back:#F0F0F0" % comment)
        obj.StyleSetSpec(wx.stc.STC_P_TRIPLE, comment)

    def attach(self, dictionary, detach=True, _internal_call=False):
        """
        Adds a dictionary to the globals and keeps track of the items so that
        they can be removed safely with the detach function. Also works for
        modules (in which case any private attributes are ignored).

        detach : bool
            Detach anything else before attaching the current dictionary.

        """
        if detach:
            self.detach()

        # detect and convert special types
        if isinstance(dictionary, types.ModuleType):
            mod = vars(dictionary)
            dictionary = {k:mod[k] for k in mod if not k.startswith('_')}

        # self._attached_items = {id(dict_like) -> {key -> value}}
        present = []
        for k in dictionary:
            if not isinstance(k, basestring):
                raise ValueError("Dictionary contains non-strong key: %r" % k)
            elif not is_py_varname(k):
                raise ValueError("Dictionary contains invalid name: %r" % k)
            elif k in self.global_namespace:
                if id(self.global_namespace[k]) != id(dictionary[k]):
                    present.append(k)

        if present:
            title = "Overwrite Items?"
            message = ("The following items are associated with different "
                       "objects in global namespace. Should we replace them?"
                       + os.linesep)
            message = os.linesep.join((message, ', '.join(repr(k) for k in present)))
            answer = ui.ask(title, message, True)
            if answer is None:
                return
            elif answer:
                attach = dictionary
            else:
                attach = dict((k, v) for k, v in dictionary.iteritems() if k not in present)
        else:
            attach = dictionary

        self.global_namespace.update(attach)

        items = self._attached_items.setdefault(id(dictionary), {})
        items.update(attach)

        msg = "attached: %s" % str(attach.keys())
        self.shell_message(msg, internal_call=_internal_call)

    def bufferSave(self):
        "to catch an distribute menu command 'save' in Os-X"
        if self.IsActive():
            wx.py.shell.ShellFrame.bufferSave(self)
        else:
            for e in self.editors:
                if e.IsActive():
                    e.bufferSave()

    def bufferSaveAs(self):
        "to catch an distribute menu command 'save as' in Os-X"
        if self.IsActive():
            wx.py.shell.ShellFrame.bufferSave(self)
        else:
            for e in self.editors:
                if e.IsActive():
                    e.bufferSaveAs()

    def CloseAllPlots(self):
        plt.close('all')
        for frame in self.Children:
            if isinstance(frame, CanvasFrame):
                frame.Close()

    def create_py_editor(self, pyfile=None, name="Script Editor"):
        """
        Creates and returns a new py_editor object.

        :arg pyfile: filename as string, or True in order to display an "open
            file" dialog (None creates an empty editor)
        :arg openfile: True if an 'open file' dialog should be shown after the
            editor is cerated

        """

        display = wx.Display()
        area = display.GetClientArea()

        shell = self.GetRect()

        w = 640
        h = area.height
        size = (w, h)

        x = min(shell.right, area.right - w)
        y = area.top
        pos = (x, y)

        editor = PyEditor(self, self, pos=pos, size=size, pyfile=pyfile,
                          name=name)

        editor.Show()
        self.editors.append(editor)

        # add to window menu
        ID = editor.GetId()
        m = self.windowMenu.Append(ID, "a", "Bring window to the front.")
        self.Bind(wx.EVT_MENU, self.OnWindowMenuActivateWindow, m)
        self.windowMenuWindows[ID] = editor
        self.windowMenuMenuItems[ID] = m

        return editor

    def curdir(self, dirname=None):
        """
        Set the current directory. With None, returns the current directory.

        """
        if dirname:
            dirname = os.path.expanduser(dirname)
            os.chdir(dirname)
            if dirname not in sys.path:
                if hasattr(self, '_added_to_path'):
                    sys.path.remove(self._added_to_path)
                sys.path.append(dirname)
                self._added_to_path = dirname
        else:
            return os.path.abspath(os.curdir)

    def detach(self, dictionary=None):
        """
        Removes the contents of `dictionary` from the global namespace. Neither
        the item itself is removed, nor are any references that were not
        created through the :func:`attach` function.

        `dictionary`: dict-like
            Dictionary which to detach; `None`: detach everything

        """
        if dictionary is None:
            dIDs = self._attached_items.keys()
        else:
            dIDs = [id(dictionary)]

        detach = {}
        for dID in dIDs:
            detach.update(self._attached_items.pop(dID))

        for k, v in detach.iteritems():
            if id(self.global_namespace[k]) == id(v):
                del self.global_namespace[k]

    def Duplicate(self, output=False):
        # make sure we have a target editor
        if not hasattr(self.active_editor, 'InsertLine'):
            self.OnFileNew()
            self.Raise()

        editor = self.active_editor

        # prepare text for transfer
        text = self.shell.GetSelectedText()
        lines = [line for line in text.split(os.linesep) if len(line) > 0]

        for line in lines:
            if output:
                editor.InsertLine(line)
            else:
                line_stripped = self.shell.lstripPrompt(line)
                if len(line_stripped) < len(line):
                    editor.InsertLine(line_stripped)

    def DuplicateFull(self):
        self.Duplicate(True)

    def ExecCommand(self, cmd):
        """Execute a single command in the shell. Can be multiline command."""
        # clean command
        cmd = fmtxt.unindent(cmd.rstrip())
        multiline = '\n' in cmd

        # insert cmd into shell and process
        self.shell.clearCommand()
        self.InsertStrToShell(cmd)
        self.shell.processLine()
        if multiline:
            self.shell.processLine()

    def ExecFile(self, filename=None, shell_globals=True):
        """
        Execute a file in the shell.

        Parameters
        ----------
        filename : None | str
            File to execute. If None, an open file dialog is used.
        shell_globals : bool
            Wheter the file should be executed in the shell's global namespace
            (or in a separate namespace).
        """
        if filename is None:
            dialog = wx.FileDialog(self, style=wx.FD_OPEN)
            dialog.SetMessage("Select Python File")
            dialog.SetWildcard("Python files (*.py)|*.py")
            if dialog.ShowModal() == wx.ID_OK:
                filename = dialog.GetPath()
            else:
                return

        if not isinstance(filename, basestring):
            err = ("Invalid filename type:  needs to be str or unicode, not "
                   "%s" % repr(filename))
            logging.error("shell.ExecFile - " + err)
            raise TypeError(err)

        if not os.path.exists(filename):
            msg = "Trying to execute non-existing file: %r" % filename
            wx.MessageBox(msg, "Error Executing File", wx.ICON_ERROR | wx.OK)
            return

        # set paths in environment
        dirname = os.path.dirname(filename)
        self.curdir(dirname)

        if shell_globals:
            self.global_namespace['__file__'] = filename
            self.shell.Execute("execfile(%r)" % filename)
        else:
            if float(sys.version[:3]) >= 2.7:
                if 'runpy' not in self.global_namespace:
                    self.shell.Execute("import runpy")
                self.shell.Execute("out_globals = runpy.run_path(%r)" % filename)
            else:
                command = "execfile(%r, dict(__file__=%r))"
                self.shell.Execute(command % (filename, filename))

        self.pyplot_draw()

    def ExecText(self, txt, out=False, title="unknown source", comment=None,
                 shell_globals=True, filepath=None, internal_call=False):
        """
        Compile txt and Execute it in the shell.

        Parameters
        ----------
        txt : str
            Code to execute.
        out : bool
            Use shell.Execute
        title:
            is displayed in the shell and should identify the source of the
            code.
        comment:
            displayed after title
        shell_globals:
            determines wheter the shell's globals are submitted to
            the call to execfile or not.
        filepath:
            Perform os.chdir and set __file__ before executing.
        """
        if comment is None:
            msg = '<exec %r>' % title
        else:
            msg = '<exec %r, %s>' % (title, comment)
        self.shell_message(msg, ascommand=False, internal_call=internal_call)

        if filepath:
            self.curdir(os.path.dirname(filepath))
            if out or shell_globals:
                self.global_namespace['__file__'] = filepath

        if out:
            self.shell.Execute(txt)
        else:
            self.shell.start_exec()

            # this does not work
#            self.shell.interp.runsource(txt)

            # prepare txt
            txt = self.shell.fixLineEndings(txt)
            txt = txt.replace("'''", '"""')
            txt += os.linesep

            # remove leading whitespaces (and comment lines)
            lines = [line for line in txt.splitlines() if not line.startswith('#')]
            ws_lead = []
            for line in lines:
                len_stripped = len(line.lstrip(' '))
                if len_stripped:
                    ws_lead.append(len(line) - len_stripped)
            rm = min(ws_lead)
            if rm:
                new_lines = []
                for line in lines:
                    new_lines.append(line[rm:] if len(line) > rm else line)
                txt = os.linesep.join(new_lines)

            # 1
            cmd = "exec(compile(r'''{txt}\n''', '{title}', 'exec'), {globals})"
            if shell_globals:
                exec_globals = "globals()"
            else:
                exec_globals = "{'__file__': %r}" % filepath

            code = cmd.format(txt=txt, title=os.path.split(title)[-1],
                              globals=exec_globals)
            self.shell.push(code, silent=True)

            # 2
#            cmp = compile(txt, os.path.split(title)[-1], 'exec')
#            self.shell.push(cmp, silent=True)

            # 3 (runs onliy first line)
#            self.shell.interp.runsource(txt)

            self.shell.end_exec()

#        self.Raise() # (causes Shell to raise above new plots)

        self.pyplot_draw()

    def FrameTable(self, table=None):
        pos = self.pos_for_new_window()
        t = TableFrame(self, table, pos=pos)
        self.tables.append(t)

    def get_active_window(self):
        "returns the active window (self, editor, help viewer, ...)"
        if self.IsActive():
            return self
        for c in self.Children:
            if hasattr(c, 'IsActive') and c.IsActive():
                return c
        for w in  wx.GetTopLevelWindows():
            if hasattr(w, 'IsActive') and w.IsActive():
                return w
        return wx.GetActiveWindow()

    def GetCurLine(self):
        return self.shell.GetCurLine()

    def hasBuffer(self):
        return True

    def help_lookup(self, what=None):
        """
        what = object whose docstring should be displayed

        """
        # this is the function that is pulled into the shell as help()
        self.OnHelpViewer(topic=what)

    def InsertStr(self, text):
        """
        Inserts the text in the active window (can be the shell or an editor)
        """
        for editor in self.editors:
            if hasattr(editor, 'IsActive') and editor.IsActive():
                editor.editor.window.ReplaceSelection(text)
                return
        self.InsertStrToShell(text)

    def InsertStrToShell(self, text):
        "Insert text into the shell's command prompt"
        # clean text
        text = text.rstrip().replace('\r', '')

        lines = text.splitlines()
        n = len(lines)
        if n == 0:
            return
        elif n > 1:
            for i in xrange(1, n):
                lines[i] = '... ' + lines[i]
        text = os.linesep.join(lines)

        if not self.shell.CanEdit():
            pos = self.shell.GetLastPosition()
            self.shell.SetSelectionStart(pos)
            self.shell.SetSelectionEnd(pos)

        self.shell.ReplaceSelection(text)

    def OnAbout(self, event):
        """Display an About window."""
        about = AboutFrame(self)
        if wx.__version__ >= '2.9':
            about.ShowWithEffect(wx.SHOW_EFFECT_BLEND)
        else:
            about.Show()
        about.SetFocus()
        return about

    def OnActivate(self, event):
        # logging.debug(" Shell Activate Event: {0}".format(event.Active))
        if hasattr(self.shell, 'Destroy'):  # if alive
            if event.Active:
                self.shell.SetCaretForeground(wx.Colour(255, 0, 0))
                self.shell.SetCaretPeriod(500)
            else:
                self.shell.SetCaretForeground(wx.Colour(200, 200, 200))
                self.shell.SetCaretPeriod(0)

    def OnClearTerminal(self, event=None):
        self.shell.clear()
        self.shell.prompt()

    def OnClose(self, event):
        # http://stackoverflow.com/a/1055506/166700
        logging.debug("WxTerm Shell OnClose")
        if event.CanVeto():
            self.Hide()
            event.Veto()
        else:
            # make sure pdb is dectivated
            n = self.shell.GetLineCount()
            line = self.shell.GetLine(n - 1)
            if line.startswith('(Pdb) '):
                self.ExecCommand('exit')

            # shut down
            self.SaveSettings()
            event.Skip()

    def OnCloseAllPlots(self, event):
        self.CloseAllPlots()

    def OnComment(self, event):
        win = self.get_active_window()
        win.Comment()

    def OnCopy(self, event):
        win = wx.Window.FindFocus()
        if hasattr(win, 'Copy'):
            win.Copy()
            return

        win = self.get_active_window()
        if hasattr(win, 'canvas'):  # matplotlib figure
            if wx.TheClipboard.Open():
                try:
                    # save temporary pdf file
                    path = tempfile.mktemp('.pdf')
                    win.canvas.figure.savefig(path)
                    # copy path
                    do = wx.FileDataObject()
                    do.AddFile(path)
                    wx.TheClipboard.SetData(do)
                finally:
                    wx.TheClipboard.Close()
        else:
            logging.debug("can't copy: %s" % win)

    def OnCopyPlus(self, event):
        win = wx.Window.FindFocus()
        if hasattr(win, 'CopyWithPrompts'):
            win.CopyWithPrompts()
            return

        win = self.get_active_window()
        if hasattr(win, 'canvas'):  # matplotlib figure
            if wx.TheClipboard.Open():
                try:
                    # save temporary pdf file
                    path = tempfile.mktemp('.png')
                    win.canvas.figure.savefig(path)
                    # copy path
                    do = wx.FileDataObject()
                    do.AddFile(path)
                    wx.TheClipboard.SetData(do)
                finally:
                    wx.TheClipboard.Close()

    def OnDuplicate(self, event):
        win = self.get_active_window()
        Id = event.GetId()
        if Id == ID.DUPLICATE_WITH_OUTPUT:
            win.DuplicateFull()
        else:
            win.Duplicate()

    def OnDestroyIcon(self, evt):
        logging.debug("DESTROY ICON CALLED")
        self.eelbrain_icon.Destroy()
        evt.Skip()

    def OnExecFile(self, event=None):
        """
        Execute a file in the shell.

        filename: if None, will ask
        isolate: execute with separate globals, do not touch the shell's
                 globals

        """
        self.ExecFile()

    def OnFileClose(self, event):
        """
        Handler to catch and distribute 'close' command  in Os-X
        (see wx.py.frame.Frame)

        """
        win = self.get_active_window()
        if win:
            try:
                win.Close()
            except:
                win.Destroy()
                raise
        else:
            event.Skip()

    def OnExecDocument(self, event):
        win = self.get_active_window()
        win.ExecDocument()

    def OnExecDocumentFromDisk(self, event):
        win = self.get_active_window()
        win.ExecDocumentFromDisk()

    def OnExecSelection(self, event):
        win = self.get_active_window()
        win.ExecSelection()

    def OnFileNew(self, event=None):
        self.create_py_editor()

    def OnFileOpen(self, event=None, path=None):
        if path is None:
            path = ui.ask_file("Open File", "Open a Python script in an "
                               "editor, or attach pickled data",
                               [('Known Files (*.py, *.pickled)',
                                 '*.py;*.pickled')])
            if not path:
                return

        if isinstance(path, basestring) and os.path.isfile(path):
            _, ext = os.path.splitext(path)
            ext = ext.lower()
            if ext == '.py':
                self.create_py_editor(pyfile=path)
            elif ext == '.pickled':
                try:
                    with open(path, 'rb') as fid:
                        dinnerplate = pickle.load(fid)
                except Exception as exc:
                    msg = '%s: %s' % (type(exc).__name__, exc)
                    sty = wx.OK | wx.ICON_ERROR
                    wx.MessageBox(msg, "Unplicking Failed", style=sty)
                    raise exc

                typename = dinnerplate.__class__.__name__
                msg = ("What name should the unpickled data of\ntype %r be assigned "
                       "to?\n(Leave empty to attach it)" % typename)
                name0 = os.path.splitext(os.path.basename(path))[0]
                dlg = wx.TextEntryDialog(self, msg, "Name for Unpickled Content", name0)
                while True:
                    if dlg.ShowModal() == wx.ID_OK:
                        name = str(dlg.GetValue()).strip()
                        if name:
                            if is_py_varname(name):
                                msg = ">>> %s = pickle.load(open(%r))" % (name, path)
                                self.shell_message(msg, internal_call=True)
                                self.global_namespace[name] = dinnerplate
                                return
                            else:
                                msg = "%r is not a valid python variable name" % name
                                edlg = wx.MessageDialog(self, msg, "Invalid name",
                                                        wx.OK | wx.ICON_ERROR)
                                edlg.ShowModal()
                        else:
                            self.shell_message(">>> attach(pickle.load(open(%r)))" % path,
                                               internal_call=True)
                            self.attach(dinnerplate, _internal_call=True)
                            return
                    else:
                        return

            else:
                statinfo = os.stat(path)
                if statinfo.st_size < 100000:
                    self.create_py_editor(path)
                else:
                    msg = ("Error: %r is no known file extension. Since the "
                           "file is bigger than 100 kb, it was not read as "
                           "plain text." % ext)
                    dlg = wx.MessageDialog(self, msg, "Error LOoding File",
                                           wx.OK | wx.ICON_ERROR)
                    dlg.ShowModal()
        else:
            msg = "No valid file path: %r" % path
            dlg = wx.MessageDialog(self, msg, "Error", wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()

    def OnFileSave(self, event):
        logging.debug("shell.OnFileSave()")
        win = self.get_active_window()
        if hasattr(win, 'bufferSave'):
            win.bufferSave()
        elif hasattr(win, 'toolbar') and hasattr(win.toolbar, 'save_figure'):
            # matplotlib figure
            win.toolbar.save_figure()
        else:
            event.Skip()

    def OnFileSaveAs(self, event):
        logging.debug("shell.OnFileSaveAs()")
        win = self.get_active_window()
        if hasattr(win, 'bufferSave'):
            win.bufferSave()
        elif hasattr(win, 'toolbar') and hasattr(win.toolbar, 'save_figure'):
            # matplotlib figure
            win.toolbar.save_figure()
        else:
            event.skip()

    def OnFocusPrompt(self, event):
        self.Show()
        self.shell.DocumentEnd()
        self.Raise()

    def OnHelp(self, event):
        "Help invoked through the Shell Menu"
        win = self.get_active_window()
        # check that the active window has a GetCurLine method
        if hasattr(win, 'GetCurLine'):
            text, pos = win.GetCurLine()
        else:
            dlg = wx.MessageDialog(self, "The active window does not have a "
                                   "GetCurLine function",
                                   "Help Call Failed", wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return

        logging.debug("Shell: help called for : %s (%i)" % (text, pos))
        self.OnHelpViewer(topic_str=text, pos=pos)
        win.Raise()

    def OnHelpExternal(self, event):
        "Called from the Help menu to open external resources"
        Id = event.GetId()
        if Id == ID.HELP_EELBRAIN:
            webbrowser.open("https://pythonhosted.org/eelbrain/")
        elif Id == ID.HELP_MPL:
            webbrowser.open("http://matplotlib.org/api/pyplot_summary.html")
        elif Id == ID.HELP_MDP:
            webbrowser.open("http://mdp-toolkit.sourceforge.net/documentation.html")
        elif Id == ID.HELP_PDB:
            webbrowser.open("http://docs.python.org/library/pdb.html")
        elif Id == ID.HELP_PYTHON:
            webbrowser.open("http://docs.python.org/")
        else:
            raise ValueError("Invalid help ID")

    def OnHelpViewer(self, event=None, topic=None, topic_str=None, pos=None):
        """
        topic :
            object
        topic_str :
            string that can be evaluated to get topic object
        pos :
            With topic_str, the pos argument can be used to search for a
            Python object name within topic_str. When the user asks for help
            in the shell or editor, the topic_str is the current line and pos
            is the caret position.

        """
        if self.help_viewer:
            if self.help_viewer.IsShown():
                self.help_viewer.Raise()
            else:
                self.help_viewer.Show(True)
        else:
#            x, y, w, h = self.GetRect().Get()
            x_min, y_min, x_max, y_max = wx.Display().GetGeometry()
            width = 650
            x_pos = x_max - width  # min(x + w, x_max-600)
            y_pos = 22
            height = y_max - y_min - y_pos - 55  # min(800, y_max-y_min-21)
            wpos = (x_pos, y_pos)
            wsize = (width, height)
            logging.debug("help viewer: size=%s, pos=%s" % (wsize, wpos))
            self.help_viewer = HelpFrame(self, size=wsize, pos=wpos)
            self.help_viewer.Show()

        if topic_str and (pos is not None):
            # TODO: use RE
#                p = re.compile('')
            # find the python command around pos in the line
            topic_scan = topic_str.replace('.', 'a')
            start = pos
            n_max = len(topic_str)
            end = min(pos, n_max)
            while (end < n_max) and is_py_char(topic_scan[end]):
                end += 1
            while is_py_char(topic_scan[start - 1]) and start > 0:
                start -= 1
            topic_str = topic_str[start:end]

        if topic_str:
            self.help_viewer.text_lookup(topic_str)
        else:
            self.help_viewer.HelpLookup(topic)

    def OnInsertColor(self, event=None):
        ctup = event.GetValue()
        mplc = tuple([round(c / 256., 3) for c in ctup[:3]])
        self.InsertStr(str(mplc))

    def OnInsertPath_File(self, event):
        self.OnInsertPath(event, t=0)

    def OnInsertPath_Dir(self, event):
        self.OnInsertPath(event, t=1)

    def OnInsertPath_New(self, event):
        self.OnInsertPath(event, t=2)

    def OnInsertPath(self, event, t=None):
        if t is None:
            t = event.GetSelection()
        # get
        if t == 0:  # File
            filenames = ui.ask_file(mult=True)
        elif t == 1:  # Dir
            filenames = ui.ask_dir()
        elif t == 2:  # New
            filenames = ui.ask_saveas(title="Get New File Name",
                                       message="Please Pick a File Name",
                                       ext=None)

        string = droptarget.filename_repr(filenames)

        if string:  # insert into terminal
            self.InsertStr(string)

    def OnMaximize(self, event=None):
        logging.debug("SHELLFRAME Maximize received")
        self.Resize(ID.SIZE_MAX)

    def OnOpenHistory(self, event):
        id_ = event.GetId()
        history = self.shell.history
        if id_ == ID.OPEN_HISTORY_CURRENT:
            for i, item in enumerate(history):
                if item.startswith(history_session_hdr % ""):
                    history = history[:i + 1]
                    break
            txt = os.linesep.join(reversed(history))
            name = "Current Session History"
        elif id_ == ID.OPEN_HISTORY:
            txt = os.linesep.join(reversed(history))
            name = "History"
        else:
            name, txt = self._history_items[id_]

        editor = self.create_py_editor(name=name)
        editor.editor.window.ReplaceSelection(txt)

    def OnOpenMneGui(self, event):
        Id = event.GetId()
        target = 'g'
        if target in self.global_namespace:
            i = 0
            name = 'g%i'
            while name % i in self.global_namespace:
                i += 1
            target = name % i

        import mne
        if (('mne' not in self.global_namespace) or
            (self.global_namespace['mne'] is not mne)):
            self.ExecCommand('import mne')

        cmd = self.mne_cmds[Id]
        cmd = "%s = mne.gui.%s()" % (target, cmd)
        self.ExecCommand(cmd)

    def OnOpenWindowMenu(self, event):
        "Updates open windows to the menu"
        menu = event.GetMenu()
#        ID = event.GetMenuId() (is always 0)
        name = menu.GetTitle()
        if name == "&Window":
            logging.debug("Window Menu Open")
            # update names
            for ID, m in self.windowMenuMenuItems.iteritems():
                window = self.windowMenuWindows[ID]
                title = window.GetTitle()
                m.SetText(title)

    def OnPreferences(self, event=None):
        if self.preferencesDialog:
            self.preferencesDialog.Raise()
        else:
            dlg = PreferencesDialog(self)
            dlg.Show()
            dlg.CenterOnScreen()
            self.preferencesDialog = dlg

    def OnPyplotDraw(self, event=None):
        plt.draw()
        if plt.get_backend() == 'WXAgg':
            plt.show()

    def OnQuit(self, event=None):
        logging.debug("WxTerm Shell OnQuit")

        # close all windows
        for w in wx.GetTopLevelWindows():
            if w is not self:
                if not w.Close():
                    return

        if self.help_viewer:
            self.help_viewer.Close()
        self.CloseAllPlots()
        ui.kill_progress_monitors()

        self.Close(force=True)

    def OnRecentItemLoad(self, event):
        fileNum = event.GetId() - wx.ID_FILE1
        logging.debug("History Load: %s" % fileNum)
        path = self.filehistory.GetHistoryFile(fileNum)
        logging.debug("History Load: %s" % path)
        self.OnFileOpen(path=path)

    def OnResize(self, event):
        Id = event.GetId()
        self.Resize(Id)

    def OnSelectColourAlt(self, event=None):
        if hasattr(self, 'c_dlg'):
            if hasattr(self.c_dlg, 'GetColourData'):
                c = self.c_dlg.GetColourData()
                c = c.GetColour()
                self.shell.ReplaceSelection(str(c))
            else:
                del self.c_dlg
        if not hasattr(self, 'c_dlg'):
            self.c_dlg = wx.ColourDialog(self)

    def OnShowExamples(self, event=None):
        """
        __init__(self, Window parent, String message=FileSelectorPromptStr,
            String defaultDir=EmptyString, String defaultFile=EmptyString,
            String wildcard=FileSelectorDefaultWildcardStr,
            long style=FD_DEFAULT_STYLE,
            Point pos=DefaultPosition) -> FileDialog
        """
        dialog = wx.FileDialog(self, "Open Eelbrain Example",
                               defaultDir=self._examples_root,
                               wildcard="Python Scripts (*.py)|*.py",
                               style=wx.FD_OPEN)
        dialog.ShowModal()
        path = dialog.GetPath()
        if path:
            self.create_py_editor(pyfile=path)

    def OnTableNew(self, event=None):
        self.FrameTable(None)

    def OnTogglePyplotMgr(self, event=None):
        if self.P_mgr.IsShown():
            self.P_mgr.Show(False)
        else:
            self.P_mgr.Show()

    def OnUpdateMenu(self, event):
        """Replace form wx.py.frame.Frame to update new menu items and
        preferences dialog.
        """
        win = self.get_active_window()
        id_ = event.GetId()
        if id_ == ID.COMMENT:
            event.Enable(hasattr(win, 'Comment'))
        elif id_ == ID.DUPLICATE:
            event.Enable(hasattr(win, 'Duplicate'))
        elif id_ == ID.DUPLICATE_WITH_OUTPUT:
            event.Enable(hasattr(win, 'DuplicateFull'))
        elif id_ == ID.EXEC_SELECTION:
            if not isinstance(event.GetEventObject(), wx.Menu):
                return
            canexec = win.CanExec() if hasattr(win, 'CanExec') else True
            event.Enable(canexec and hasattr(win, 'ExecSelection'))
        elif id_ == ID.EXEC_DOCUMENT:
            if not isinstance(event.GetEventObject(), wx.Menu):
                return
            canexec = win.CanExec() if hasattr(win, 'CanExec') else True
            event.Enable(canexec and hasattr(win, 'ExecDocument'))
        elif id_ == ID.EXEC_DOCUMENT_FROM_DISK:
            if not isinstance(event.GetEventObject(), wx.Menu):
                return
            canexec = win.CanExec() if hasattr(win, 'CanExec') else True
            event.Enable(canexec and hasattr(win, 'ExecDocumentFromDisk'))
        else:
            if id_ == ID_SAVEHISTORY and self.preferencesDialog:
                state = event.IsChecked()
                self.preferencesDialog.checkboxSaveHistory.SetValue(state)
            elif id_ == ID_AUTO_SAVESETTINGS and self.preferencesDialog:
                state = event.IsChecked()
                self.preferencesDialog.checkboxSaveSettings.SetValue(state)
            elif (id_ == wx.ID_COPY and hasattr(win, 'canvas') and
                  hasattr(win.canvas, 'figure') and
                  hasattr(win.canvas.figure, 'savefig')):  # matplotlib figure
                event.Enable(True)
                return
            elif id_ == ID_COPY_PLUS:
                if (hasattr(win, 'canvas') and hasattr(win.canvas, 'figure')
                    and hasattr(win.canvas.figure, 'savefig')):  # matplotlib figure
                    event.Enable(True)
                    event.SetText("Cop&y as PNG \tCtrl+Shift+C")
                    return
                else:
                    event.SetText("Cop&y Plus \tCtrl+Shift+C")
            super(ShellFrame, self).OnUpdateMenu(event)

    def OnWindowMenuActivateWindow(self, event):
        ID = event.GetId()
        window = self.windowMenuWindows[ID]
        window.Show()
        window.Raise()

    def pos_for_new_window(self, size=(200, 400)):
        x, y, w, h = self.GetRect().Get()
        x_min, y_min, x_max, y_max = wx.Display().GetGeometry()
        e_x = x + w
        if e_x + size[0] > x_max:
            e_x = x_max - size[0]
        return (e_x, y_min + 26)

    def pyplot_draw(self):
        # update plots if mpl Backend does not do that automatically
        if mpl.get_backend() in ['WXAgg']:
            if len(plt._pylab_helpers.Gcf.get_all_fig_managers()) > 0:
                plt.draw()
                plt.show()

    def recent_item_add(self, path, t):
        "t is 'e' or 'pydoc'"
        self.filehistory.AddFileToHistory(path)
        self.filehistory.Save(self.config)
        self.config.Flush()
        self.recent_menu_update_icons()

#        path_list = self.recent_files[t]
#        menu = self.recent_menus[t]
#
#        if path in path_list:
#            i = path_list.index(path)
#            if i == 0:
#                return
#            path_list.pop(i)
#            menu_item = menu.GetMenuItems()[i]
#            menu.RemoveItem(menu_item)
#        else:
#            # remove old item
#            if len(path_list) > 10:
#                path_list.pop(-1)
#                n = menu.GetMenuItemCount()
#                old_menu_item = menu.GetMenuItems()[n-1]
#                menu.RemoveItem(old_menu_item)
#            # new item
#            if t == 'e':
#                id = ID.RECENT_LOAD_E
#            elif t =='pydoc':
#                id = ID.RECENT_LOAD_PYDOC
#            else:
#                raise ValueError("t == %s"%t)
#            name = os.path.basename(path)
#            menu_item = wx.MenuItem(menu, id, name, path)
#        menu.InsertItem(0, menu_item)
#        # path list
#        path_list.insert(0, path)
#        with open(self.recent_files_path, 'w') as file:
#            pickle.dump(self.recent_files, file)
    def recent_menu_update_icons(self):
        for item in self.recent_menu.GetMenuItems():
            text = item.GetItemLabelText()
            if text.endswith('.py'):
                item.SetBitmap(Icon('documents/pydoc'))
            elif text.endswith('startup'):
                item.SetBitmap(Icon('documents/pydoc-startup'))
            else:
                item.SetBitmap(Icon('documents/unknown'))

    def RemovePyEditor(self, editor):
        if editor in self.editors:
            self.editors.remove(editor)
        ID = editor.GetId()
        self.windowMenuWindows.pop(ID)
        self.windowMenuMenuItems.pop(ID)
        self.windowMenu.Remove(ID)

    def Resize(self, Id=ID.SIZE_MAX):
        self.Show()
        display = wx.Display()
        area = display.GetClientArea()
        if Id == ID.SIZE_MAX:
            width = min(area.width // 2, 700)
            rect = wx.Rect(area.left, area.top, width, area.height)
        elif Id == ID.SIZE_MAX_NOTITLE:
            width = min(area.width // 2, 700)
            rect = wx.Rect(area.left, 0, width, area.height + area.top)
        elif Id == ID.SIZE_MIN:
            rect = wx.Rect(area.left, 200, 350, 600)
        self.SetRect(rect)

    def set_debug_mode(self, redirect, logfile=None, write=True):
        """Change the debug mode (whether to show internal exceptions)

        Parameters
        ----------
        redirect : bool
            Redirect output for internal internal exceptions.
        logfile : None | str
            A file to redirect the output to. If None, open a window with
            the output.
        write : bool
            Save the new settings to the configuration file.

        Notes
        -----
        Turning off debug mode only takes effect after restarting the
        Application.
        """
        if write:
            self.config.WriteBool('Debug/Redirect', redirect)
            self.config.Write('Debug/Logfile', logfile or '')

        if redirect:
            app = wx.GetApp()
            app.RedirectStdio(logfile)

    def shell_message(self, message, sep=False, ascommand=False, endline=True,
                      internal_call=False):
        """
        Proxy for shell.writeOut method

        kwargs
        ------
        sep = False: adds linebreaks at the top and at the bottom of message

        ascommand = False: adds the prompt in front of the message to mimmick
                   command

        internal_call: notification that the call is made by the app mainloop
                       rather than form the shell

        """
        ls = os.linesep
        if message != ls:
            message = message.rstrip(ls)

        if ascommand:
            message = sys.ps1 + message
            endline = True

        if endline:
            if internal_call:
                message = message + ls
        elif sep:
            if message[0] != ls:
                message = ls + message
            if message[-1] != ls:
                message = message + ls
            if message[-2] != ls:
                message = message + ls

        if internal_call:  # ascommand:
            logging.debug("internal shell_msg: %r" % message)
            self.shell.start_exec()
            self.shell.writeOut(message)
            self.shell.end_exec()
        else:
            logging.debug("external shell_msg: %r" % message)
            print message




import logging, tempfile, os

import matplotlib.pyplot as P
import wx

from .. import fmtxt
from .._utils import ui
from .._wxutils import Icon


class PyplotManager(wx.MiniFrame):
    copy_as_file = True  # copy as file or use mpl copy function
    def __init__(self, parent, pos=wx.DefaultPosition):
        wx.MiniFrame.__init__(self, parent, -1, "PyplotManager",
                              pos=pos,  # wx.Point
                              size=wx.Size(100, 500),
                              style=wx.DEFAULT_FRAME_STYLE)

        panel = self.panel = wx.Panel(self, -1)
#        sizer = self.sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer = self.sizer = wx.GridBagSizer(3, 2)
        panel.SetSizer(sizer)

#        button = wx.Button(panel, -1, "Update")
#        sizer.Add(button, 0, wx.EXPAND)
        button = wx.BitmapButton(panel, -1, Icon("tango/actions/view-refresh"))
        sizer.Add(button, (0, 0))
        button.Bind(wx.EVT_BUTTON, self.OnUpdate)

#        button = wx.Button(panel, -1, "Close All")
#        sizer.Add(button, 0, wx.EXPAND)
        button = wx.BitmapButton(panel, -1, Icon("tango/status/image-missing"))
        sizer.Add(button, (0, 1))
        button.Bind(wx.EVT_BUTTON, self.OnCloseAll)

        # ## SELECT
        ch = self.select = wx.Choice(panel, -1, size=(100, -1),
                                     choices=["Select"],
                                     name="Select")
        sizer.Add(ch, (1, 0), (1, 2))
        ch.Bind(wx.EVT_LEFT_DOWN, self.update_copy_fignums)
        ch.Bind(wx.EVT_CHOICE, self.OnFigureSelect)
        # ##

        # ## COPY
        ch = self.copy = wx.Choice(panel, -1, size=(100, -1),
                                   choices=["Copy"],
                                   name="Copy")
        sizer.Add(ch, (2, 0), (1, 2))
        ch.Bind(wx.EVT_LEFT_DOWN, self.update_copy_fignums)
        ch.Bind(wx.EVT_CHOICE, self.OnFigureSelect)
        # ##

        button = wx.BitmapButton(panel, -1, Icon("copy/textab"))
        sizer.Add(button, (3, 0))
        button.Bind(wx.EVT_BUTTON, self.OnCopyTextab)


        sizer.Fit(panel)
        self.Bind(wx.EVT_CLOSE, self.OnCloseWindow)

        x, y = sizer.GetMinSize()
        x += 10
        y += 20
        self.SetSize((x, y))  # x was 100


#    def figure(self, num=None, **kwargs):
#        """
#        could provide function to the shell to create a customized figure
#        frame in the future
#
#        """
#        if num is None:
#            num = 1
#            used = self.figures.keys()
#            while num in used:
#                num += 1
#        else:
#            if num in self.figures:
#                return self.figures[num]
#        # if not found create new figure
#        fig = self.P_figure(num, **kwargs)
#        self.figures[num] = fig
#
#        self.update_figurelist()
#        return fig

    def update_copy_fignums(self, event):
        sender = event.GetEventObject()
        allnums = [str(f.num) for f in P._pylab_helpers.Gcf.get_all_fig_managers()]
        sender.SetItems([sender.Name] + allnums)
        event.Skip()

    def OnFigureSelect(self, event):
        sender = event.GetEventObject()
        choice = event.GetString()
        if choice.isdigit():
            num = int(choice)
            if sender == self.select:
                P.figure(num)
            elif sender == self.copy:
                fig = P.figure(num)
                if self.copy_as_file:
                    if wx.TheClipboard.Open():
                        # save to temp file
                        fd, path = tempfile.mkstemp('.pdf', text=True)
#                        os.write(fd, pdf)
                        os.close(fd)
                        logging.debug("Temporary file created at: %s" % path)
                        fig.savefig(path)
                        # copy path
            #            shutil.copyfileobj(f, 'bar.txt')
                        do = wx.FileDataObject()
                        do.AddFile(path)
                        wx.TheClipboard.SetData(do)
                        wx.TheClipboard.Close()


                else:
                    fig.set_facecolor((1, 1, 1))
                    P.draw()
                    canvas = fig.canvas
                    if hasattr(canvas, 'Copy_to_Clipboard'):
                        canvas.Copy_to_Clipboard()
                    else:
                        logging.warning("Could Not Copy Figure %s To ClipBoard" % num)
            else:
                raise ValueError("URK!")
            sender.SetSelection(0)

    def OnUpdate(self, event):
        P.draw()
        if P.get_backend() == 'WXAgg':
            P.show()

    def OnCopyTextab(self, event=None):
        try:
            fmtxt.copy_pdf()
        except Exception as e:
            ui.message("Error in Copy Tex", str(e), '!')

    def OnCloseAll(self, event):
        "Close all open figures"
        self.Parent.CloseAllPlots()

    def OnCloseWindow(self, event):
        "Hides the window instead of destroying it"
        logging.debug("P_Mgr.OnCloseWindow()")
        self.Show(False)
#        self.Destroy()



'''
Created on Dec 2, 2012

@author: christian
'''
from eelbrain import Factor, datasets, plot, testnd


def test_stat():
    "test plot.UTSStat plotting function"
    plot.configure_backend(False, False)
    ds = datasets.get_rand()
    p = plot.UTSStat('uts', ds=ds)
    p.close()
    p = plot.UTSStat('uts', 'A%B', ds=ds)
    p.close()
    p = plot.UTSStat('uts', 'A', Xax='B', ds=ds)
    p.close()
    p = plot.UTSStat('uts', 'A%B', 'rm', sub="rm.isin(('R00', 'R01'))", ds=ds)
    p.close()

    # clusters
    sds = ds.sub("B == 'b0'")
    res = testnd.ttest_rel('uts', 'A', 'a1', 'a0', match='rm', ds=sds,
                           samples=0, pmin=0.05, mintime=0.02)
    p = plot.UTSStat('uts', 'A', clusters=res.clusters, ds=ds)
    p.close()
    res = testnd.ttest_rel('uts', 'A', 'a1', 'a0', match='rm', ds=sds,
                           samples=100, pmin=0.05, mintime=0.02)
    p = plot.UTSStat('uts', 'A', clusters=res.clusters, ds=ds)
    p.close()


def test_uts():
    "test plot.UTS plotting function"
    plot.configure_backend(False, False)
    ds = datasets.get_rand()
    p = plot.UTS('uts', ds=ds)
    p.close()
    p = plot.UTS('uts', 'A%B', ds=ds)
    p.close()


def test_clusters():
    "test plot.uts cluster plotting functions"
    plot.configure_backend(False, False)
    ds = datasets.get_rand()

    A = ds['A']
    B = ds['B']
    Y = ds['uts']

    # fixed effects model
    res = testnd.anova(Y, A * B)
    p = plot.UTSClusters(res, title="Fixed Effects Model")
    p.close()

    # random effects model:
    subject = Factor(range(15), tile=4, random=True, name='subject')
    res = testnd.anova(Y, A * B * subject, samples=2)
    p = plot.UTSClusters(res, title="Random Effects Model")
    p.close()

    # plot UTSStat
    p = plot.UTSStat(Y, A % B, match=subject)
    p.set_clusters(res.clusters)
    p.close()
    p = plot.UTSStat(Y, A, Xax=B, match=subject)
    p.close()



"""Test mne interaction"""
from itertools import izip
import os

from nose.tools import (assert_equal, assert_less_equal, assert_not_equal,
                        assert_true, assert_in)
import numpy as np
from numpy.testing import assert_array_equal

import mne

from eelbrain import datasets, load, testnd, morph_source_space
from eelbrain._data_obj import asndvar, SourceSpace

from .test_data import assert_dataobj_equal

# mne paths
data_dir = mne.datasets.sample.data_path()
subjects_dir = os.path.join(data_dir, 'subjects')


def connectivity_from_coo(coo):
    """Convert a coo matrix to Eelbrain internal connectivity

    Returns
    -------
    connetivity : array of int, (n_pairs, 2)
        array of sorted [src, dst] pairs, with all src < dts.
    """
    pairs = set()
    for v0, v1, d in izip(coo.row, coo.col, coo.data):
        if not d or v0 == v1:
            continue
        src = min(v0, v1)
        dst = max(v0, v1)
        pairs.add((src, dst))
    connectivity = np.array(sorted(pairs), dtype=np.int32)
    return connectivity


def test_source_estimate():
    "Test SourceSpace dimension"
    mne.set_log_level('warning')
    ds = datasets.get_mne_sample(src='ico')
    dsa = ds.aggregate('side')

    # test auto-conversion
    asndvar('epochs', ds=ds)
    asndvar('epochs', ds=dsa)
    asndvar(dsa['epochs'][0])

    # source space clustering
    res = testnd.ttest_ind('src', 'side', ds=ds, samples=0, pmin=0.05,
                           tstart=0.05, mintime=0.02, minsource=10)
    assert_equal(res.clusters.n_cases, 52)

    # test disconnecting parc
    src = ds['src']
    source = src.source
    parc = source.parc
    orig_conn = set(map(tuple, source.connectivity()))
    disc_conn = set(map(tuple, source.connectivity(True)))
    assert_true(len(disc_conn) < len(orig_conn))
    for pair in orig_conn:
        s, d = pair
        if pair in disc_conn:
            assert_equal(parc[s], parc[d])
        else:
            assert_not_equal(parc[s], parc[d])

    # threshold-based test with parc
    srcl = src.sub(source='lh')
    res = testnd.ttest_ind(srcl, 'side', ds=ds, samples=10, pmin=0.05,
                           tstart=0.05, mintime=0.02, minsource=10,
                           parc='source')
    assert_equal(res._cdist.dist.shape[1], len(srcl.source.parc.cells))
    label = 'superiortemporal-lh'
    c_all = res._clusters(maps=True)
    c_label = res._clusters(maps=True, source=label)
    assert_array_equal(c_label['location'], label)
    for case in c_label.itercases():
        id_ = case['id']
        idx = c_all['id'].index(id_)[0]
        assert_equal(case['v'], c_all[idx, 'v'])
        assert_equal(case['tstart'], c_all[idx, 'tstart'])
        assert_equal(case['tstop'], c_all[idx, 'tstop'])
        assert_less_equal(case['p'], c_all[idx, 'p'])
        assert_dataobj_equal(case['cluster'],
                             c_all[idx, 'cluster'].sub(source=label))

    # threshold-free test with parc
    res = testnd.ttest_ind(srcl, 'side', ds=ds, samples=10, tstart=0.05,
                           parc='source')
    cl = res._clusters(0.05)
    assert_equal(cl.eval("p.min()"), res.p.min())
    mp = res.masked_parameter_map()
    assert_in(mp.min(), (0, res.t.min()))
    assert_in(mp.max(), (0, res.t.max()))

    # indexing source space
    s_sub = src.sub(source='fusiform-lh')
    idx = source.index_for_label('fusiform-lh')
    s_idx = src[idx]
    assert_dataobj_equal(s_sub, s_idx)


def test_morphing():
    mne.set_log_level('warning')
    sss_path = os.path.join(subjects_dir, 'fsaverage', 'bem', 'fsaverage-ico-4-src.fif')
    sss = mne.read_source_spaces(sss_path, False)
    vertices_to = [sss[0]['vertno'], sss[1]['vertno']]
    ds = datasets.get_mne_sample(-0.1, 0.1, src='ico', sub='index==0', stc=True)
    stc = ds['stc', 0]
    morph_mat = mne.compute_morph_matrix('sample', 'fsaverage', stc.vertno,
                                         vertices_to, None, subjects_dir)
    ndvar = ds['src']

    morphed_ndvar = morph_source_space(ndvar, 'fsaverage')
    morphed_stc = mne.morph_data_precomputed('sample', 'fsaverage', stc,
                                             vertices_to, morph_mat)
    assert_array_equal(morphed_ndvar.x[0], morphed_stc.data)
    morphed_stc_ndvar = load.fiff.stc_ndvar([morphed_stc], 'fsaverage', 'ico-4',
                                            subjects_dir, 'src', parc=None)
    assert_dataobj_equal(morphed_ndvar, morphed_stc_ndvar)


def test_source_space():
    "Test SourceSpace dimension"
    for subject in ['fsaverage', 'sample']:
        path = os.path.join(subjects_dir, subject, 'bem', subject + '-ico-4-src.fif')
        mne_src = mne.read_source_spaces(path)
        vertno = [mne_src[0]['vertno'], mne_src[1]['vertno']]
        ss = SourceSpace(vertno, subject, 'ico-4', subjects_dir, 'aparc')

        # connectivity
        conn = ss.connectivity()
        mne_conn = mne.spatial_src_connectivity(mne_src)
        assert_array_equal(conn, connectivity_from_coo(mne_conn))

        # sub-space connectivity
        sssub = ss[ss.dimindex('superiortemporal-rh')]
        ss2 = SourceSpace(vertno, subject, 'ico-4', subjects_dir, 'aparc')
        ss2sub = ss2[ss2.dimindex('superiortemporal-rh')]
        assert_array_equal(sssub.connectivity(), ss2sub.connectivity())



# Author: Christian Brodbeck <christianbrodbeck@nyu.edu>
import numpy as np
from matplotlib import pyplot as plt

from eelbrain import Factor, Var, datasets, plot


def test_barplot():
    "Test plot.uv.barplot"
    plot.configure_backend(False, False)
    ds = datasets.get_uv()
    plot.uv.Barplot('fltvar', 'A%B', match='rm', ds=ds)
    plot.uv.Barplot('fltvar', 'A%B', match='rm', test=0, ds=ds)
    plt.close('all')


def test_boxplot():
    "Test plot.uv.boxplot"
    plot.configure_backend(False, False)
    ds = datasets.get_uv()
    plot.uv.Boxplot('fltvar', 'A%B', match='rm', ds=ds)
    plt.close('all')

    # many pairwise significances
    ds['fltvar'][ds.eval("A%B==('a1','b1')")] += 1
    ds['fltvar'][ds.eval("A%B==('a2','b2')")] -= 1
    ds['C'] = Factor('qw', rep=10, tile=4)
    plot.uv.Boxplot('fltvar', 'A%B%C', ds=ds)


def test_histogram():
    "Test plot.uv.histogram"
    plot.configure_backend(False, False)
    ds = datasets.get_rand()
    plot.uv.Histogram('Y', 'A%B', ds=ds)
    plot.uv.Histogram('Y', 'A%B', match='rm', ds=ds)
    plt.close('all')


def test_scatterplot():
    "Test plot.uv.corrplot and lot.uv.regplot"
    plot.configure_backend(False, False)
    ds = datasets.get_rand()
    ds['cov'] = ds['Y'] + np.random.normal(0, 1, (60,))

    plot.uv.Correlation('Y', 'cov', ds=ds)
    plot.uv.Correlation('Y', 'cov', 'A%B', ds=ds)

    plot.uv.Regression('Y', 'cov', ds=ds)
    plot.uv.Regression('Y', 'cov', 'A%B', ds=ds)

    plt.close('all')


def test_timeplot():
    "Test plot.uv.timeplot"
    plot.configure_backend(False, False)
    ds = datasets.get_rand()
    ds['seq'] = Var(np.arange(2).repeat(30))
    plot.uv.Timeplot('Y', 'B', 'seq', match='rm', ds=ds)
    plt.close('all')




'''
Created on Dec 2, 2012

@author: christian
'''
from eelbrain import datasets, table, combine


def test_difference():
    "Test tabe.difference"
    ds = datasets.get_uv()
    print table.difference('fltvar', 'A', 'a1', 'a2', 'rm', ds=ds)
    print table.difference('fltvar', 'A', 'a1', 'a2', 'rm', by='B', ds=ds)
    print table.difference('fltvar', 'A%B', ('a1', 'b1'), ('a2', 'b2'), 'rm',
                           ds=ds)

    # create bigger dataset
    ds['C', :] = 'c1'
    ds2 = datasets.get_uv()
    ds2['C', :] = 'c2'
    ds = combine((ds, ds2))
    print table.difference('fltvar', 'A', 'a1', 'a2', 'rm', 'B%C', ds=ds)
    print table.difference('fltvar', 'A%B', ('a1', 'b1'), ('a2', 'b2'), 'rm',
                           'C', ds=ds)


def test_frequencies():
    "test table.frequencies"
    ds = datasets.get_rand()
    A = ds['A']
    B = ds['B']
    Cat = ds['YCat']
    print table.frequencies(Cat, A)
    print table.frequencies(Cat, A % B)
    print table.frequencies(Cat % A, B)


def test_repmeas():
    "Test table.repmeas (repeated measures table)"
    ds = datasets.get_uv()
    print table.repmeas('fltvar', 'A', 'rm', ds=ds)
    print table.repmeas('fltvar', 'A%B', 'rm', ds=ds)



'''Eelbrain GUIs'''

def run():
    "Hand over command to the GUI (quit the GUI to return to the terminal)"
    from . import _wxgui
    _wxgui.run()

def select_epochs(*args, **kwargs):
    """GUI for rejecting trials

    Parameters
    ----------
    ds : Dataset | mne.Epochs
        The data for which to select trials. If ds is an mne.Epochs object
        the subsequent parameters up to 'trigger' are irrelevant.
    data : str
        Name of the epochs data in ds.
    accept : str
        Name of the boolean Var in ds to accept or reject epochs.
    blink : str
        Name of the eye tracker data in ds.
    tag : str
        Name of the rejection tag (storing the reason for rejecting a
        specific epoch).
    trigger : str
        Name of the int Var containing the event trigger for each epoch
        (used to assert that when loading a rejection file it comes from
        the same data).
    path : None | str
        Path to the desired rejection file. If the file already exists it
        is loaded as starting values. The extension determines the format
        (*.pickled or *.txt).
    nplots : None | int | tuple of 2 int
        Number of epoch plots per page. Can be an ``int`` to produce a
        square layout with that many epochs, or an ``(n_rows, n_columns)``
        tuple. Default (None): use settings form last session.
    topo : None | bool
        Show a topomap plot of the time point under the mouse cursor.
        Default (None): use settings form last session.
    mean : None | bool
        Show a plot of the page mean at the bottom right of the page.
        Default (None): use settings form last session.
    vlim : None | scalar
        Limit of the epoch plots on the y-axis. If None, a value is
        determined automatically to show all data.
    plot_range : bool
        In the epoch plots, plot the range of the data (instead of plotting
        all sensor traces). This makes drawing of pages quicker, especially
        for data with many sensors (default ``True``).
    color : None | matplotlib color
        Color for primary data (default is black).
    lw : scalar
        Linewidth for normal sensor plots.
    mark : None | index for sensor dim
        Sensors to plot as individual traces with a separate color.
    mcolor : matplotlib color
        Color for marked traces.
    mlw : scalar
        Line width for marked sensor plots.
    antialiased : bool
        Perform Antialiasing on epoch plots (associated with a minor speed
        cost).
    pos : None | tuple of 2 int
        Window position on screen. Default (None): use settings form last
        session.
    size : None | tuple of 2 int
        Window size on screen. Default (None): use settings form last
        session.

    Notes
    -----
    In addition to the menus, the following keyboard shortcuts are available:

        right-arrow:
            Go to next page.
        left-arrow:
            Go to previous page.
        b:
            Butterfly plot of the currently displayed epoch.
        c:
            Pairwise sensor correlation plot or the current epoch.
        t:
            Topomap plot of the currently displayed time point.
        u:
            Undo.
        shift-u:
            Redo.
    """
    from . import _wxgui
    _wxgui.select_epochs.TerminalInterface(*args, **kwargs)



'''
Created on Dec 2, 2012

@author: christian
'''
from itertools import izip
import os
import cPickle as pickle
import shutil
from string import ascii_lowercase
import tempfile

import mne
from nose.tools import (assert_equal, assert_almost_equal, assert_is_instance,
                        assert_true, eq_, ok_, assert_raises)
import numpy as np
from numpy.testing import assert_array_equal, assert_array_almost_equal

from eelbrain import (datasets, load, Var, Factor, Dataset, Celltable,
                      align, align1, combine)
from eelbrain._data_obj import (asvar, isdatalist, isndvar, isvar, isuv,
                                Categorial, SourceSpace, UTS)
from eelbrain._stats import rms


def assert_dataset_equal(ds1, ds2, msg="Datasets unequal", decimal=None):
    """
    Raise an assertion if two Datasets are not equal up to desired precision.

    Parameters
    ----------
    ds1, ds2 : Dataset
        Datasets to compare.
    msg : str
        Prefix of the error message to be printed in case of failure.
    decimal : None | int
        Desired precision (default is exact match).
    """
    assert_equal(ds1.keys(), ds2.keys(), "%s: different keys (%s vs %s)" %
                 (msg, ds1.keys(), ds2.keys()))
    for k in ds1.keys():
        assert_dataobj_equal(ds1[k], ds2[k], msg=msg, decimal=decimal)
    assert_equal(ds1.info.keys(), ds2.info.keys(), "%s: keys in info" % msg)


def assert_dataobj_equal(d1, d2, msg="Data-objects unequal", decimal=None):
    """
    Raise an assertion if two data-objects are not equal up to desired
    precision.

    Parameters
    ----------
    ds1, ds2 : data-objects
        Data-objects to compare.
    msg : str
        Prefix of the error message to be printed in case of failure.
    decimal : None | int
        Desired precision (default is exact match).
    """
    msg = "%s:" % msg
    assert_equal(d1.name, d2.name, "%s unequal names (%r vs %r"
                 ")" % (msg, d1.name, d2.name))
    msg += 'Two objects named %r have' % d1.name
    len1 = len(d1)
    len2 = len(d2)
    assert_equal(len1, len2, "%s unequal length: %i/%i" % (msg, len1, len2))
    if isvar(d1) and decimal:
        assert_array_almost_equal(d1.x, d2.x, decimal)
    elif isuv(d1):
        assert_true(np.all(d1 == d2), "%s unequal values: %r vs "
                    "%r" % (msg, d1, d2))
    elif isndvar(d1):
        assert_true(np.all(d1.x == d2.x), "%s unequal values" % msg)
    elif isdatalist(d1):
        for i in xrange(len(d1)):
            assert_equal(d1[i], d2[i], "%s unequal values" % msg)


def assert_source_space_equal(src1, src2, msg="SourceSpace Dimension objects "
                              "unequal"):
    """
    Raise an assertion if two SourceSpace objects are not equal up to desired
    precision.

    Parameters
    ----------
    src1, src2 : SourceSpace objects
        SourceSpace objects to compare.
    msg : str
        Prefix of the error message to be printed in case of failure.
    """
    msg = "%s:" % msg
    assert_array_equal(src1.vertno[0], src2.vertno[0], "%s unequal lh vertno "
                       "(%r vs %r)" % (msg, src1.vertno[0], src2.vertno[0]))
    assert_array_equal(src1.vertno[1], src2.vertno[1], "%s unequal rh vertno "
                       "(%r vs %r)" % (msg, src1.vertno[1], src2.vertno[1]))
    assert_equal(src1.subject, src2.subject, "%s unequal subject (%r vs %r"
                 ")" % (msg, src1.subject, src2.subject))
    assert_equal(src1.src, src2.src, "%s unequal names (%r vs %r"
                 ")" % (msg, src1.src, src2.src))
    assert_equal(src1.subjects_dir, src2.subjects_dir, "%s unequal names (%r "
                 "vs %r)" % (msg, src1.subjects_dir, src2.subjects_dir))


def test_print():
    "Run the string representation methods"
    ds = datasets.get_rand()
    print ds
    print repr(ds)
    A = ds['A']
    print A
    print repr(A)
    Y = ds['Y']
    print Y
    print repr(Y)
    Ynd = ds['uts']
    print Ynd
    print repr(Ynd)


def test_aggregate():
    "Test aggregation methods"
    ds = datasets.get_rand()

    # don't handle inconsistencies silently
    assert_raises(ValueError, ds.aggregate, 'A%B')

    dsa = ds.aggregate('A%B', drop_bad=True)
    assert_array_equal(dsa['n'], [15, 15, 15, 15])
    idx1 = ds.eval("logical_and(A=='a0', B=='b0')")
    assert_equal(dsa['Y', 0], ds['Y', idx1].mean())

    # unequal cell counts
    ds = ds[:-3]
    dsa = ds.aggregate('A%B', drop_bad=True)
    assert_array_equal(dsa['n'], [15, 15, 15, 12])
    idx1 = ds.eval("logical_and(A=='a0', B=='b0')")
    assert_equal(dsa['Y', 0], ds['Y', idx1].mean())

    dsa = ds.aggregate('A%B', drop_bad=True, equal_count=True)
    assert_array_equal(dsa['n'], [12, 12, 12, 12])
    idx1_12 = np.logical_and(idx1, idx1.cumsum() <= 12)
    assert_equal(dsa['Y', 0], ds['Y', idx1_12].mean())


def test_align():
    "Testing align() and align1() functions"
    ds = datasets.get_uv()
    ds.index()
    idx4 = np.arange(0, ds.n_cases, 4)
    idx4i = idx4[::-1]
    ds2 = ds.sub(np.arange(0, ds.n_cases, 2))

    # align1: align Dataset to index
    dsa = align1(ds2, idx4)
    assert_array_equal(dsa['index'], idx4, "align1() failure")
    dsa = align1(ds2, idx4i)
    assert_array_equal(dsa['index'], idx4i, "align1() failure")
    # d_idx as Var
    dsa = align1(ds2[::2], idx4, idx4i)
    assert_array_equal(dsa['index'], idx4i, "align1() failure")
    assert_raises(ValueError, align1, ds2, idx4, idx4i)

    # Factor index
    assert_raises(ValueError, align1, ds, ds['rm', ::-1], 'rm')
    fds = ds[:20]
    dsa = align1(fds, fds['rm', ::-1], 'rm')
    assert_array_equal(dsa['index'], np.arange(19, -1, -1), "align1 Factor")

    # align two datasets
    dsa1, dsa2 = align(ds, ds2)
    assert_array_equal(dsa1['index'], dsa2['index'], "align() failure")
    dsa1, dsa2 = align(ds, ds2[::-1])
    assert_array_equal(dsa1['index'], dsa2['index'], "align() failure")


def test_celltable():
    "Test the Celltable class."
    ds = datasets.get_rand()
    ds['cat'] = Factor('abcd', rep=15)

    ct = Celltable('Y', 'A', ds=ds)
    eq_(ct.n_cases, 60)
    eq_(ct.n_cells, 2)

    ct = Celltable('Y', 'A', match='rm', ds=ds)
    eq_(ct.n_cases, 30)
    eq_(ct.n_cells, 2)

    ct = Celltable('Y', 'cat', cat=('c', 'b'), ds=ds)
    eq_(ct.n_cases, 30)
    eq_(ct.X[0], 'c')
    eq_(ct.X[-1], 'b')

    ct = Celltable('Y', 'A', match='rm', ds=ds)
    eq_(ct.n_cases, 30)
    assert np.all(ct.groups['a0'] == ct.groups['a1'])

    ct = Celltable('Y', 'cat', match='rm', cat=('c', 'b'), ds=ds)
    eq_(ct.n_cases, 30)
    eq_(ct.X[0], 'c')
    eq_(ct.X[-1], 'b')

    # coercion of numerical X
    X = ds.eval("A == 'a0'")
    ct = Celltable('Y', X, cat=(None, None), ds=ds)
    assert_equal(('False', 'True'), ct.cat)
    assert_array_equal(ct.data['True'], ds['Y', X])

    ct = Celltable('Y', X, cat=(True, False), ds=ds)
    assert_equal(('True', 'False'), ct.cat)
    assert_array_equal(ct.data['True'], ds['Y', X])

    # test coercion of Y
    ct = Celltable(ds['Y'].x, 'A', ds=ds)
    assert_is_instance(ct.Y, np.ndarray)
    ct = Celltable(ds['Y'].x, 'A', ds=ds, coercion=asvar)
    assert_is_instance(ct.Y, Var)

    # test sub
    ds_sub = ds.sub("A == 'a0'")
    ct_sub = Celltable('Y', 'B', ds=ds_sub)
    ct = Celltable('Y', 'B', sub="A == 'a0'", ds=ds)
    assert_dataobj_equal(ct_sub.Y, ct.Y)

    # test sub with rm
    ct_sub = Celltable('Y', 'B', match='rm', ds=ds_sub)
    ct = Celltable('Y', 'B', match='rm', sub="A == 'a0'", ds=ds)
    assert_dataobj_equal(ct_sub.Y, ct.Y)

    # test rm sorting
    ds = Dataset()
    ds['rm'] = Factor('abc', rep=4)
    ds['Y'] = Var(np.arange(3.).repeat(4))
    ds['X'] = Factor('ab', rep=2, tile=3)
    idx = np.arange(12)
    np.random.shuffle(idx)
    ds = ds[idx]
    ct = Celltable('Y', 'X', 'rm', ds=ds)
    assert_array_equal(ct.match, Factor('abc', tile=2))
    assert_array_equal(ct.Y, np.tile(np.arange(3.), 2))
    assert_array_equal(ct.X, Factor('ab', rep=3))


def test_combine():
    "Test combine()"
    ds1 = datasets.get_rand()
    ds2 = datasets.get_rand()
    ds = combine((ds1, ds2))
    assert_array_equal(ds2['Y'].x, ds['Y'].x[ds1.n_cases:], "Basic combine")
    del ds1['Y']
    del ds2['YCat']
    ds = combine((ds1, ds2))
    assert_array_equal(ds2['Y'].x, ds['Y'].x[ds1.n_cases:], "Combine with "
                       "missing Var")
    assert_true(np.all(ds1['YCat'] == ds['YCat'][:ds1.n_cases]), "Combine "
                "with missing Factor")

    # combine NDVar with unequel dimensions
    ds = datasets.get_rand(utsnd=True)
    y = ds['utsnd']
    y1 = y.sub(sensor=['0', '1', '2', '3'])
    y2 = y.sub(sensor=['1', '2', '3', '4'])
    ds1 = Dataset(y1)
    ds2 = Dataset(y2)
    dsc = combine((ds1, ds2))
    y = dsc['utsnd']
    assert_equal(y.sensor.names, ['1', '2', '3'], "Sensor dimension "
                 "intersection failed.")
    dims = ('case', 'sensor', 'time')
    ref = np.concatenate((y1.get_data(dims)[:, 1:], y2.get_data(dims)[:, :3]))
    assert_array_equal(y.get_data(dims), ref, "combine utsnd")


def test_dataset_combining():
    "Test Dataset combination methods"
    ds = datasets.get_uv()
    del ds['fltvar'], ds['intvar'], ds['A']

    ds2 = datasets.get_uv()
    del ds2['fltvar'], ds2['intvar']
    ds.update(ds2)
    assert_array_equal(ds['A'], ds2['A'])

    ds2 = datasets.get_uv()
    del ds2['fltvar'], ds2['intvar']
    ds2['B'][5] = 'something_else'
    del ds['A']
    assert_raises(ValueError, ds.update, ds2)


def test_dataset_indexing():
    """Test Dataset indexing"""
    ds = datasets.get_uv()

    # indexing values
    assert_equal(ds['A', 1], ds['A'][1])
    assert_equal(ds[1, 'A'], ds['A'][1])

    # indexing variables
    assert_dataobj_equal(ds[:, 'A'], ds['A'])
    assert_dataobj_equal(ds['A', :], ds['A'])
    assert_dataobj_equal(ds[:10, 'A'], ds['A'][:10])
    assert_dataobj_equal(ds['A', :10], ds['A'][:10])

    # new Dataset through indexing
    ds2 = Dataset()
    ds2['A'] = ds['A']
    assert_dataset_equal(ds[('A',)], ds2)
    ds2['B'] = ds['B']
    assert_dataset_equal(ds['A', 'B'], ds2)
    assert_dataset_equal(ds[('A', 'B'), :10], ds2[:10])
    assert_dataset_equal(ds[:10, ('A', 'B')], ds2[:10])

    # assigning value
    ds[2, 'A'] = 'hello'
    assert_equal(ds[2, 'A'], 'hello')
    ds['A', 2] = 'not_hello'
    assert_equal(ds[2, 'A'], 'not_hello')

    # assigning new factor
    ds['C', :] = 'c'
    ok_(np.all(ds.eval("C == 'c'")))

    # assigning new Var
    ds['D1', :] = 5.
    ds[:, 'D2'] = 5.
    assert_array_equal(ds['D1'], 5)
    assert_array_equal(ds['D2'], 5)

    # test illegal names
    f = Factor('aaabbb')
    assert_raises(ValueError, ds.__setitem__, '%dsa', f)
    assert_raises(ValueError, ds.__setitem__, '432', f)
    assert_raises(ValueError, ds.__setitem__, ('%dsa', slice(None)), 'value')
    assert_raises(ValueError, ds.__setitem__, (slice(None), '%dsa'), 'value')
    assert_raises(ValueError, ds.__setitem__, ('432', slice(None)), 4.)
    assert_raises(ValueError, ds.__setitem__, (slice(None), '432'), 4.)


def test_dataset_sorting():
    "Test Dataset sorting methods"
    test_array = np.arange(10)
    ds = Dataset()
    ds['v'] = Var(test_array)
    ds['f'] = Factor(test_array)

    # shuffle the Dataset
    rand_idx = test_array.copy()
    np.random.shuffle(rand_idx)
    ds_shuffled = ds[rand_idx]

    # ascending, Var, copy
    dsa = ds_shuffled.sorted('v')
    assert_dataset_equal(dsa, ds, "Copy sorted by Var, ascending")

    # descending, Factor, in-place
    ds_shuffled.sort('f', descending=True)
    assert_dataset_equal(ds_shuffled, ds[::-1], "In-place sorted by Factor, "
                         "descending")


def test_dim_categorial():
    "Test Categorial Dimension"
    values = ['a', 'b', 'c', 'abc']
    name = 'cat'
    dim = Categorial(name, values)

    # basic properties
    print dim
    assert_equal(len(dim), len(values))

    # persistence
    s = pickle.dumps(dim, pickle.HIGHEST_PROTOCOL)
    dim_ = pickle.loads(s)
    assert_equal(dim_, dim)

    # indexing
    sub_values = values[:2]
    idx = dim.dimindex(sub_values)
    assert_array_equal(dim.dimindex(tuple(sub_values)), idx)
    assert_equal(dim[idx], Categorial(name, sub_values))
    assert_equal(dim.dimindex('a'), values.index('a'))
    assert_equal(dim.dimindex('abc'), values.index('abc'))

    # intersection
    dim2 = Categorial(name, ['c', 'b', 'e'])
    dim_i = dim.intersect(dim2)
    assert_equal(dim_i, Categorial(name, ['b', 'c']))

    # unicode
    dimu = Categorial(name, [u'c', 'b', 'e'])
    assert_equal(dimu.values.dtype.kind, 'U')
    assert_equal(dim2.values.dtype.kind, 'S')
    assert_equal(dimu, dim2)


def test_dim_uts():
    "Test UTS Dimension"
    uts = UTS(-0.1, 0.005, 301)

    # make sure indexing rounds correctly for floats
    for i, s in enumerate(np.arange(0, 1.4, 0.05)):
        idx = uts.dimindex((-0.1 + s, s))
        assert_equal(idx.start, 10 * i)
        assert_equal(idx.stop, 20 + 10 * i)

    # intersection
    uts1 = UTS(-0.1, 0.01, 50)
    uts2 = UTS(0, 0.01, 20)
    intersection = uts1.intersect(uts2)
    assert_equal(intersection, uts2)
    idx = uts1.dimindex((0, 0.2))
    assert_equal(uts1[idx], uts2)


def test_effect():
    "Test _Effect class"
    # .enumerate_cells()
    f1 = Factor('aabbccaabbcc')
    f2 = Factor('abababababab')
    i = f1 % f2

    n1 = np.concatenate((np.tile([0, 1], 3), np.tile([2, 3], 3)))
    assert_array_equal(f1.enumerate_cells(), n1)
    assert_array_equal(f2.enumerate_cells(), np.arange(6).repeat(2))
    assert_array_equal(i.enumerate_cells(), np.arange(2).repeat(6))


def test_isin():
    "Test .isin() methods"
    values = np.array([  6, -6, 6, -2, -1, 0, -10, -5, -10, -6])
    v = values[0]
    v2 = values[:2]
    labels = {i: c for i, c in enumerate(ascii_lowercase, -10)}
    vl = labels[v]
    v2l = [labels[v_] for v_ in v2]

    target = np.logical_or(values == v2[0], values == v2[1])
    inv_target = np.invert(target)
    index_target = np.flatnonzero(values == v)
    empty = np.array([])

    var = Var(values)
    assert_array_equal(var.index(v), index_target)
    assert_array_equal(var.isin(v2), target)
    assert_array_equal(var.isany(*v2), target)
    assert_array_equal(var.isnot(*v2), inv_target)
    assert_array_equal(var.isnotin(v2), inv_target)

    var0 = Var([])
    assert_array_equal(var0.isin(v2), empty)
    assert_array_equal(var0.isany(*v2), empty)
    assert_array_equal(var0.isnot(*v2), empty)
    assert_array_equal(var0.isnotin(v2), empty)

    f = Factor(values, labels=labels)
    assert_array_equal(f.index(vl), index_target)
    assert_array_equal(f.isin(v2l), target)
    assert_array_equal(f.isany(*v2l), target)
    assert_array_equal(f.isnot(*v2l), inv_target)
    assert_array_equal(f.isnotin(v2l), inv_target)

    f0 = Factor([])
    assert_array_equal(f0.isin(v2l), empty)
    assert_array_equal(f0.isany(*v2l), empty)
    assert_array_equal(f0.isnot(*v2l), empty)
    assert_array_equal(f0.isnotin(v2l), empty)


def test_ndvar():
    "Test the NDVar class"
    ds = datasets.get_rand(utsnd=True)
    x = ds['utsnd']

    # slicing
    assert_raises(KeyError, x.sub, sensor='5')
    assert_equal(x.sub(sensor='4').ndim, 2)
    assert_equal(x.sub(sensor=['4']).ndim, 3)
    assert_equal(x.sub(case=1, sensor='4').ndim, 1)

    # baseline correction
    x_bl = x - x.summary(time=(None, 0))
    # assert that the baseline is 0
    bl = x_bl.summary('case', 'sensor', time=(None, 0))
    ok_(abs(bl) < 1e-10, "Baseline correction")

    # NDVar as index
    sens_mean = x.mean(('case', 'time'))
    idx = sens_mean > 0
    pos = sens_mean[idx]
    assert_array_equal(pos.x > 0, True)


def test_ndvar_summary_methods():
    "Test NDVar methods for summarizing data over axes"
    ds = datasets.get_rand(utsnd=True)
    x = ds['utsnd']

    dim = 'sensor'
    axis = x.get_axis(dim)
    dims = ('case', 'sensor')
    axes = tuple(x.get_axis(d) for d in dims)
    idx = x > 0
    x0 = x[0]
    idx0 = idx[0]
    xsub = x.sub(time=(0, 0.5))
    idxsub = xsub > 0
    idx1d = x.mean(('case', 'time')) > 0

    # numpy functions
    assert_equal(x.any(), x.x.any())
    assert_array_equal(x.any(dim), x.x.any(axis))
    assert_array_equal(x.any(dims), x.x.any(axes))
    assert_array_equal(x.any(idx0), [x_[idx0.x].any() for x_ in x.x])
    assert_array_equal(x.any(idx), [x_[i].any() for x_, i in izip(x.x, idx.x)])
    assert_array_equal(x0.any(idx0), x0.x[idx0.x].any())
    assert_array_equal(x.any(idxsub), xsub.any(idxsub))
    assert_array_equal(x.any(idx1d), x.x[:, idx1d.x].any(1))

    assert_equal(x.max(), x.x.max())
    assert_array_equal(x.max(dim), x.x.max(axis))
    assert_array_equal(x.max(dims), x.x.max(axes))
    assert_array_equal(x.max(idx0), [x_[idx0.x].max() for x_ in x.x])
    assert_array_equal(x.max(idx), [x_[i].max() for x_, i in izip(x.x, idx.x)])
    assert_array_equal(x0.max(idx0), x0.x[idx0.x].max())
    assert_array_equal(x.max(idxsub), xsub.max(idxsub))
    assert_array_equal(x.max(idx1d), x.x[:, idx1d.x].max(1))

    assert_equal(x.mean(), x.x.mean())
    assert_array_equal(x.mean(dim), x.x.mean(axis))
    assert_array_equal(x.mean(dims), x.x.mean(axes))
    assert_array_equal(x.mean(idx0), [x_[idx0.x].mean() for x_ in x.x])
    assert_array_equal(x.mean(idx), [x_[i].mean() for x_, i in izip(x.x, idx.x)])
    assert_array_equal(x0.mean(idx0), x0.x[idx0.x].mean())
    assert_array_equal(x.mean(idxsub), xsub.mean(idxsub))
    assert_array_equal(x.mean(idx1d), x.x[:, idx1d.x].mean(1))

    assert_equal(x.min(), x.x.min())
    assert_array_equal(x.min(dim), x.x.min(axis))
    assert_array_equal(x.min(dims), x.x.min(axes))
    assert_array_equal(x.min(idx0), [x_[idx0.x].min() for x_ in x.x])
    assert_array_equal(x.min(idx), [x_[i].min() for x_, i in izip(x.x, idx.x)])
    assert_array_equal(x0.min(idx0), x0.x[idx0.x].min())
    assert_array_equal(x.min(idxsub), xsub.min(idxsub))
    assert_array_equal(x.min(idx1d), x.x[:, idx1d.x].min(1))

    assert_equal(x.std(), x.x.std())
    assert_array_equal(x.std(dim), x.x.std(axis))
    assert_array_equal(x.std(dims), x.x.std(axes))
    assert_array_equal(x.std(idx0), [x_[idx0.x].std() for x_ in x.x])
    assert_array_equal(x.std(idx), [x_[i].std() for x_, i in izip(x.x, idx.x)])
    assert_array_equal(x0.std(idx0), x0.x[idx0.x].std())
    assert_array_equal(x.std(idxsub), xsub.std(idxsub))
    assert_array_equal(x.std(idx1d), x.x[:, idx1d.x].std(1))

    # non-numpy
    assert_equal(x.rms(), rms(x.x))
    assert_array_equal(x.rms(dim), rms(x.x, axis))
    assert_array_equal(x.rms(dims), rms(x.x, axes))
    assert_array_equal(x.rms(idx0), [rms(x_[idx0.x]) for x_ in x.x])
    assert_array_equal(x.rms(idx), [rms(x_[i]) for x_, i in izip(x.x, idx.x)])
    assert_array_equal(x0.rms(idx0), rms(x0.x[idx0.x]))
    assert_array_equal(x.rms(idxsub), xsub.rms(idxsub))
    assert_array_equal(x.rms(idx1d), rms(x.x[:, idx1d.x], 1))


def test_ols():
    "Test NDVar.ols() method"
    from rpy2.robjects import r

    # simulate data
    ds = datasets.get_rand(True)
    n_times = len(ds['uts'].time)
    x = np.zeros(n_times)
    x[20:40] = np.hanning(20)
    utsc = ds.eval("uts.copy()")
    utsc.x += ds['Y'].x[:, None] * x[None, :]
    ds_ = Dataset()
    ds_['x'] = Var(ds['Y'].x)
    ds_['x2'] = ds_['x'] + np.random.normal(0, 1, ds.n_cases)

    # ols regression
    m1 = ds_['x']
    b1 = utsc.ols(m1)
    res1 = utsc.residuals(m1)
    m2 = ds_.eval("x + x2")
    b2 = utsc.ols(m2)
    res2 = utsc.residuals(m2)
    # compare with R
    for i in xrange(n_times):
        ds_['y'] = Var(utsc.x[:, i])
        ds_.to_r('ds')
        # 1 predictor
        r('lm1 <- lm(y ~ x, ds)')
        beta = r('coef(lm1)')[1]
        assert_almost_equal(b1.x[0, i], beta)
        res = r('residuals(lm1)')
        assert_array_almost_equal(res1.x[:, i], res)
        # 2 predictors
        r('lm2 <- lm(y ~ x + x2, ds)')
        beta = r('coef(lm2)')[1:]
        assert_array_almost_equal(b2.x[:, i], beta)
        res = r('residuals(lm2)')
        assert_array_almost_equal(res2.x[:, i], res)

    # 3d
    utsnd = ds['utsnd']
    ds_['utsnd'] = utsnd
    b1 = ds_.eval("utsnd.ols(x)")
    res1 = ds_.eval("utsnd.residuals(x)")
    for i in xrange(len(b1.time)):
        ds_['y'] = Var(utsnd.x[:, 1, i])
        ds_.to_r('ds')
        # 1 predictor
        r('lm1 <- lm(y ~ x, ds)')
        beta = r('coef(lm1)')[1]
        assert_almost_equal(b1.x[0, 1, i], beta)
        res = r('residuals(lm1)')
        assert_array_almost_equal(res1.x[:, 1, i], res)


def test_io_pickle():
    "Test io by pickling"
    ds = datasets.get_rand()
    ds.info['info'] = "Some very useful information about the Dataset"
    tempdir = tempfile.mkdtemp()
    try:
        dest = os.path.join(tempdir, 'test.pickled')
        with open(dest, 'wb') as fid:
            pickle.dump(ds, fid, protocol=pickle.HIGHEST_PROTOCOL)
        with open(dest, 'rb') as fid:
            ds2 = pickle.load(fid)
    finally:
        shutil.rmtree(tempdir)

    assert_dataset_equal(ds, ds2)


def test_io_txt():
    "Test Dataset io as text"
    ds = datasets.get_uv()

    # Var that has integer values as float
    ds['intflt'] = ds.eval('intvar * 1.')
    ds['intflt'].name = 'intflt'

    # io test
    tempdir = tempfile.mkdtemp()
    try:
        dest = os.path.join(tempdir, 'test.txt')
        ds.save_txt(dest)
        ds2 = load.tsv(dest)
    finally:
        shutil.rmtree(tempdir)

    assert_dataset_equal(ds, ds2, decimal=6)


def test_r():
    "Test interaction with R thorugh rpy2"
    from rpy2.robjects import r

    r("data(sleep)")
    ds = Dataset.from_r("sleep")
    assert_equal(ds.name, 'sleep')
    extra = (0.7, -1.6, -0.2, -1.2, -0.1, 3.4, 3.7, 0.8, 0.0, 2.0, 1.9, 0.8,
             1.1, 0.1, -0.1, 4.4, 5.5, 1.6, 4.6, 3.4)
    assert_array_equal(ds.eval('extra'), extra)
    assert_array_equal(ds.eval('ID'), map(str, xrange(1, 11)) * 2)
    assert_array_equal(ds.eval('group'), ['1'] * 10 + ['2'] * 10)

    # test putting
    ds.to_r('sleep_copy')
    ds_copy = Dataset.from_r('sleep_copy')
    assert_dataset_equal(ds_copy, ds)


def test_source_space():
    "Test SourceSpace Dimension"
    subject = 'fsaverage'
    data_path = mne.datasets.sample.data_path()
    mri_sdir = os.path.join(data_path, 'subjects')
    mri_dir = os.path.join(mri_sdir, subject)
    src_path = os.path.join(mri_dir, 'bem', subject + '-ico-5-src.fif')
    label_dir = os.path.join(mri_dir, 'label')
    label_ba1 = mne.read_label(os.path.join(label_dir, 'lh.BA1.label'))
    label_v1 = mne.read_label(os.path.join(label_dir, 'lh.V1.label'))
    label_mt = mne.read_label(os.path.join(label_dir, 'lh.MT.label'))
    label_ba1_v1 = label_ba1 + label_v1
    label_v1_mt = label_v1 + label_mt

    src = mne.read_source_spaces(src_path)
    source = SourceSpace((src[0]['vertno'], src[1]['vertno']), subject,
                         'ico-5', mri_sdir)
    index = source.dimindex(label_v1)
    source_v1 = source[index]
    index = source.dimindex(label_ba1_v1)
    source_ba1_v1 = source[index]
    index = source.dimindex(label_v1_mt)
    source_v1_mt = source[index]
    index = source_ba1_v1.dimindex(source_v1_mt)
    source_v1_intersection = source_ba1_v1[index]
    assert_source_space_equal(source_v1, source_v1_intersection)

    # index from label
    index = source.index_for_label(label_v1)
    assert_array_equal(index.source[index.x].vertno[0],
                       np.intersect1d(source.lh_vertno, label_v1.vertices, 1))

    # parcellation and cluster localization
    if mne.__version__ < '0.8':
        return
    parc = mne.read_labels_from_annot(subject, parc='aparc', subjects_dir=mri_sdir)
    indexes = [source.index_for_label(label) for label in parc
               if len(label) > 10]
    x = np.vstack([index.x for index in indexes])
    ds = source._cluster_properties(x)
    for i in xrange(ds.n_cases):
        assert_equal(ds[i, 'location'], parc[i].name)


def test_var():
    "Test Var objects"
    base = Factor('aabbcde')
    Y = Var.from_dict(base, {'a': 5, 'e': 8}, default=0)
    assert_array_equal(Y.x, [5, 5, 0, 0, 0, 0, 8])

    # .split()
    y = Var(np.arange(16))
    for i in xrange(1, 9):
        split = y.split(i)
        assert_equal(len(split.cells), i)



# Author: Christian Brodbeck <christianbrodbeck@nyu.edu>
'''
Statistics functions that work on numpy arrays.

'''

import numpy as np
import scipy.stats

__all__ = ['ci', 'cihw', 'rms', 'rmssd', 'ftest_f', 'ftest_p']


def ci(x, p=.95):
    """
    :returns: list with the endpoints of the confidence interval based on the
        inverse t-test
        (`<http://en.wikipedia.org/wiki/Confidence_interval#Statistical_hypothesis_testing>`_).

    :arg array x: data
    :arg float p: p value for confidence interval

    """
    M = np.mean(x)
    c = cihw(x, p)
    return [M - c, M + c]


def cihw(x, p=.95):
    """
    :returns: half-width of the confidence interval based on the inverse t-test
        (`<http://en.wikipedia.org/wiki/Confidence_interval#Statistical_hypothesis_testing>`_).

    :arg array x: data
    :arg float p: p value for confidence interval

    """
    N = len(x)
    t = scipy.stats.t.isf((1 - p) / 2, N - 1)
    c = (np.std(x, ddof=1) * t) / np.sqrt(N)
    return c


def rms(a, axis=None):
    """Root mean square

    Parameters
    ----------
    a : array_like
        Data.
    axis : None | int | tuple of ints
        Axis or axes over which to calculate the RMS.
        The default (`axis` = `None`) is the RMS over all the dimensions of
        the input array.
    """
    square = np.square(a)
    out = square.mean(axis)
    if np.isscalar(out):
        return np.sqrt(out)
    else:
        return np.sqrt(out, out)


def rmssd(Y):
    """Root mean square of successive differences

    Used for heart rate variance analysis.
    """
    assert np.ndim(Y) == 1
    N = len(Y)
    assert N > 1
    dY = np.diff(Y)
    X = N / (N - 1) * np.sum(dY ** 2)
    return np.sqrt(X)


def ftest_f(p, df_num, df_den):
    "F values for given probabilities."
    p = np.asanyarray(p)
    f = scipy.stats.f.isf(p, df_num, df_den)
    return f


def ftest_p(f, df_num, df_den):
    "P values for given f values."
    f = np.asanyarray(f)
    p = scipy.stats.f.sf(f, df_num, df_den)
    return p



"""Some basic statistical tests for univariate data.

.. autosummary::
   :toctree: generated

   anova
   oneway
   pairwise
   ttest
   correlations
   lilliefors

"""
__test__ = False

from ._stats.glm import anova
from ._stats.test import (pairwise, ttest, oneway, correlations,
                          bootstrap_pairwise, lilliefors)



'''
About dialog for Eelbrain wx-terminal


Created on Feb 20, 2012

@author: christian
'''

import wx
from wx.lib.agw import hyperlink

from .. import __version__
from .._wxutils import Icon


class AboutFrame(wx.MiniFrame):
    def __init__(self, parent):
        wx.MiniFrame.__init__(self, parent, -1, pos=(100, 100),
                              style=wx.CLOSE_BOX)

        sizer = wx.BoxSizer(wx.VERTICAL)
        self.SetSizer(sizer)

    # LOGO / name / version
        img = Icon('eelbrain160')
        bmp = wx.StaticBitmap(self, -1, img)
        sizer.Add(bmp, 0, wx.LEFT | wx.TOP | wx.RIGHT | wx.ALIGN_CENTER_HORIZONTAL, 10)

        txt = "Eelbrain"
        text = wx.StaticText(self, -1, txt)
        font = text.GetFont()
        font.SetPointSize(30)
        text.SetFont(font)
        sizer.Add(text, 0, wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER_HORIZONTAL, 10)

        txt = "Version %s" % __version__
        text = wx.StaticText(self, -1, txt)
        text.SetForegroundColour(wx.Colour(120, 120, 120))
        sizer.Add(text, 0, wx.LEFT | wx.BOTTOM | wx.RIGHT | wx.ALIGN_CENTER_HORIZONTAL, 10)


    # Links
        link = hyperlink.HyperLinkCtrl(self, wx.ID_ANY, "Eelbrain Documentation",
                                       URL="https://pythonhosted.org/eelbrain/")
        sizer.Add(link, 0, wx.LEFT | wx.TOP | wx.RIGHT | wx.ALIGN_CENTER_HORIZONTAL, 10)

        link = hyperlink.HyperLinkCtrl(self, wx.ID_ANY, "Eelbrain on GitHub",
                                       URL="https://github.com/christianbrodbeck/Eelbrain/")
        sizer.Add(link, 0, wx.LEFT | wx.TOP | wx.RIGHT | wx.ALIGN_CENTER_HORIZONTAL, 10)


    # Text
        txt = (u"Eelbrain is \xa9 Christian M Brodbeck\n"
               u"all open-source components are\n\xa9 their owners")
        text = wx.StaticText(self, -1, txt, style=wx.ALIGN_CENTER)
#        txt = (u"Eelbrain is \xa9 Christian Brodbeck; All open-source "
#               u"components are \xa9 their owners.")
#        text = wx.TextCtrl(self, -1, txt,
#                           style = wx.TE_READONLY|wx.TE_MULTILINE)
#        text.SetMinSize(wx.Size(250, 50))
        self.text = text
        sizer.Add(text, 0, wx.ALL | wx.EXPAND, 10)


        # buttons
        h_sizer = wx.BoxSizer(wx.HORIZONTAL)

        button = wx.Button(self, -1, "About PyShell")
        self.Bind(wx.EVT_BUTTON, self.OnAboutPyShell, button)
        h_sizer.Add(button, 0, wx.ALL | wx.ALIGN_LEFT, 10)

        button = wx.Button(self, -1, "Close")
        self.Bind(wx.EVT_BUTTON, self.OnClose, button)
        h_sizer.Add(button, 0, wx.ALL | wx.ALIGN_RIGHT, 10)
        sizer.Add(h_sizer, 0)
#        sizer.Add(button, 0, wx.ALL|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_BOTTOM, 10)

        self.Center(wx.CENTER_ON_SCREEN | wx.HORIZONTAL)
        self.Fit()

    def OnClose(self, event):
        self.Close()

    def OnAboutPyShell(self, event):
        wx.py.shell.ShellFrame.OnAbout(self.Parent, event)



from nose.tools import assert_raises

from eelbrain.plot._base import Layout


def assert_layout_ok(*args, **kwargs):
    error = None
    l = Layout(*args, **kwargs)
    if l.nrow * l.ncol < l.nax:
        error = ("%i rows * %i cols = %i < %i (nax). args=%%r, kwargs=%%r"
                 % (l.nrow, l.ncol, l.nrow * l.ncol, l.nax))

    if error:
        raise AssertionError(error % (args, kwargs))


def test_layout():
    "Test the Layout class"
    for nax in xrange(1, 100):
        assert_layout_ok(nax, ax_aspect=1.5, axh_default=2, w=5)
        assert_layout_ok(nax, ax_aspect=1.5, axh_default=2, h=5)
        assert_layout_ok(nax, ax_aspect=1.5, axh_default=2, axw=5)
        assert_layout_ok(nax, ax_aspect=1.5, axh_default=2, axh=5)
        assert_layout_ok(nax, ax_aspect=1.5, axh_default=2, axw=5, w=20)
        assert_layout_ok(nax, ax_aspect=1.5, axh_default=2, axw=5, h=20)
        assert_layout_ok(nax, ax_aspect=1.5, axh_default=2, axh=5, w=20)
        assert_layout_ok(nax, ax_aspect=1.5, axh_default=2, axh=5, h=20)

    # single axes larger than figure
    assert_raises(ValueError, Layout, 2, h=5, axh=6)
    assert_raises(ValueError, Layout, 2, w=5, axw=6)




def launch(py2app=False):
    "launches Eelbrain's wxPython terminal"
    import wx
    if wx.GetApp():
        raise RuntimeError("wx.App already running")
        # prevent irregular behavior if called from Eelbrain or pyshell

    import matplotlib as mpl
    mpl.use('WXAgg')

    # configure the logging module so it logs debug messages
    # On OS X they can be viewed in the Console
    import logging
    logging.basicConfig(level=logging.DEBUG)

    from .app import MainApp

    app = MainApp(py2app)
    app.MainLoop()



'''Communication utilities'''
import bz2
from email.mime.text import MIMEText
import os
import pdb
import smtplib
import socket
import traceback

from .basic import logger


_pwd_fname = os.path.expanduser('~/.eelbrain_n00b')


def send_email(to, subject, body):
    """Send an email notification"""
    gmail_user = 'n00b.eelbrain@gmail.com'
    with open(_pwd_fname) as f:
        pwd = bz2.decompress(f.read())

    msg = MIMEText(body)
    msg['Subject'] = subject
    host = socket.gethostname()
    msg['From'] = 'Eelbrain on %s <%s>' % (host, gmail_user)
    msg['To'] = to

    smtpserver = smtplib.SMTP('smtp.gmail.com', 587)
    smtpserver.ehlo()
    smtpserver.starttls()
    smtpserver.login(gmail_user, pwd)

    smtpserver.sendmail(gmail_user, to, msg.as_string())
    smtpserver.close()


class Notifier(object):
    """
    A notification email sender supporting ``with`` statements

    Examples
    --------
    To receive a message after a task has been executed:

    >>> notifier = Notifier('me@somewhere.com')
    >>> with notifier:
    ...     do_task()
    ...

    """
    def __init__(self, to, name='job', state_func=None, debug=True):
        """
        Parameters
        ----------
        to : str
            Email address of the recipient.
        name : str
            Name of the job (will be included in subject line).
        state_func : None | callable
            Will be called upon crash to produce a string that will be included
            in the crash report.
        debug : bool
            If the task crashes, start pdb instead of exiting.
        """
        if not os.path.exists(_pwd_fname):
            err = "File required for notification not found: %r" % _pwd_fname
            raise IOError(err)

        self.to = to
        self.name = name
        self.state_func = state_func
        self.debug = debug

    def __enter__(self):
        logger.info("Notification enabled...")
        return self

    def __exit__(self, type_, value, traceback_):
        host = socket.gethostname()
        if isinstance(value, Exception):
            error = type_.__name__
            temp = '{host} encountered {error}: {value} in {task}'
            event = temp.format(host=host, error=error, value=value,
                                task=self.name)
            info = []

            # state description
            if self.state_func:
                state_desc = self.state_func()
                info.append(state_desc)

            # traceback
            tb_items = traceback.format_tb(traceback_)
            tb_str = '\n'.join(tb_items)
            info.append(tb_str)

            # error
            info.append("%s: %s" % (error, value))

            if self.debug:
                info.append("Starting PDB...")
            else:
                info.append("Terminating...")

            self.send(event, info)

            # drop into pdb
            if self.debug:
                traceback.print_exc()
                pdb.post_mortem(traceback_)
        else:
            event = '{host} finished {task}'.format(host=host, task=self.name)
            self.send(event)

    def send(self, subject, info=[]):
        """Send an email message

        Parameters
        ----------
        subject : str
            Email subject line.
        info : list of str
            Email body; successive entries are joined with two line breaks.
        """
        body = '\n\n'.join(map(unicode, info))
        send_email(self.to, subject, body)



"""
Module to deploy GUI elements depending on the current environment.

"""

import logging
import os

try:
    import wx
    import wx_ui
except ImportError:
    logging.info("wx unavailable; using tk_ui")
    wx_ui = False


__all__ = ['ask', 'ask_color', 'ask_dir', 'ask_saveas', 'ask_str',
           'copy_file'
           'message',
           'progress_monitor', 'kill_progress_monitors',
           'test_targetpath',
           ]

_progress_monitors = []


def get_ui():
    if wx_ui and bool(wx.GetApp()):
        return wx_ui
    else:
        import tk_ui
        return tk_ui


def ask_saveas(title="Save File", message="Please Pick a File Name",
               filetypes=[("Pickled Python Objects (*.pickled)", '*.pickled')],
               defaultDir=None, defaultFile=False):
    """Display a save-as dialog

    Parameters
    ----------
    title : str
        Title of the dialog.
    message : str
        Message in the dialog.
    filetypes : sequence of tuples
        Sequence of (label, pattern) tuples.
    defaultDir : None | str
        Default directory to save at.
    defaultFile : None | str
        Default file name.

    Returns
    -------
    result : False | str
        The path as str if the user selects a dialog, otherwise ``False``.
    """
    result = get_ui().ask_saveas(title, message, filetypes, defaultDir,
                                 defaultFile)
    return result


def ask_dir(title="Select Folder",
            message="Please Pick a Folder",
            must_exist=True):
    return get_ui().ask_dir(title, message, must_exist)


def ask_file(title="Pick File",
             message="Please Pick a File",
             filetypes=[("All files", '*')],
             directory='',
             mult=False):
    """
    Ask for an existing file.

    Parameters
    ----------
    title, message : str
        Title and message for the dialog.
    filetypes : sequence of tuples
        Sequence of (label, pattern) tuples.
    directory : None | str
        Path to initial directory.
    mult : bool
        Allow selecting multiple files.

    Returns
    -------
    paths : False | str | list
        If the user cancels: False. Otherwise, if mult=False a single path, and
        if mult=True a list of paths.
    """
    return get_ui().ask_file(title, message, filetypes, directory, mult)


def ask(title="Overwrite File?",
        message="Duplicate filename. Do you want to overwrite?",
        cancel=False,
        default=True,  # True=YES, False=NO, None=Nothing
        ):
    return get_ui().ask(title, message, cancel, default)


def ask_color(default=(0, 0, 0)):
    return get_ui().ask_color(default)


def ask_str(msg, title, default=''):
    return get_ui().ask_str(msg, title, default)


def kill_progress_monitors():
    while len(_progress_monitors) > 0:
        p = _progress_monitors.pop()
        p.terminate()

def show_help(obj):
    return get_ui().show_help(obj)


def message(title, message="", icon='i'):
    return get_ui().message(title, message, icon)


def progress_monitor(i_max=None,
                     title="Task Progress",
                     message="Good luck!",
                     cancel=True):
    """
    With wx-python, show a progress dialog. In Tk this is not fully
    implemented and merely sends messages to logging.info.

    i_max : None | int
        Final state; None: show an indeterminate progress bar.
    title : str
        The window title.
    message : str
        The message; is replaced when :py:meth:`advance` is called with a
        str argument.
    cancel : bool
        Show a cancel button. If the 'cancel' button is clicked,
        the next call to :py:meth`advance` will raise a KeyboardInterrupt
        Exception.

    """
    dialog = get_ui().progress_monitor(i_max=i_max, title=title, message=message,
                                       cancel=cancel)
    _progress_monitors.append(dialog)
    return dialog


def copy_file(path):
    return get_ui().copy_file(path)


def copy_text(text):
    return get_ui().copy_text(text)


def test_targetpath(path, cancel=True):
    """
    Returns True if path is a valid path to write to, False otherwise. If the
    directory does not exist, the user is asked whether it should be created.

    cancel : bool
        Add a cancel button. If clicked, a KeyboardInterrupt Exception is
        raised.

    """
    if not path:
        return False

    dirname = os.path.abspath(os.path.dirname(path))
    if not os.path.exists(dirname):
        msg = ("The directory %r does not exist. Should it be created?" % dirname)
        answer = ask("Create Directory?", msg, cancel=cancel)
        if answer:
            os.makedirs(dirname)
        elif answer is None:  # cancel
            err = ("User canceled because the directory %r does not exist"
                   % dirname)
            raise KeyboardInterrupt(err)

    return os.path.exists(dirname)



# -*- coding: utf-8 -*-
from nose.tools import assert_equal
import os
import shutil
import tempfile

from eelbrain import fmtxt
from eelbrain.fmtxt import html, tex
from eelbrain import datasets, plot


def test_symbol():
    "Test fmtxt.symbol()"
    s = fmtxt.symbol('t', 21)
    assert_equal(str(s), 't(21)')
    assert_equal(html(s), 't<sub>21</sub>')


def test_fmtext():
    "Tet FMText base class"
    t = fmtxt.FMText('test')
    print t

    tuc = fmtxt.FMText(u'FMText with unicode: \x80abc')
    print str(tuc)

    ts = fmtxt.FMText((t, tuc, u'unicode: \x80abc'))

    print str(ts)
    print html(ts)
    print tex(ts)


def test_report():
    "Test fmtxt.Report class"
    tempdir = tempfile.mkdtemp()
    report = fmtxt.Report("Test Report")

    section = report.add_section(u'unicode: \xe2 abc')
    ds = datasets.get_uv()
    p = plot.uv.barplot('fltvar', 'A', sub="B=='b1'", ds=ds)
    image = p.image()
    section.add_figure("test", image)

    report.sign()

    # report output
    print report
    dst = os.path.join(tempdir, 'report.html')
    report.save_html(dst)

    # clean up
    shutil.rmtree(tempdir)
